"""
Python module generated from Java source file java.io.File

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import *
from java.net import MalformedURLException
from java.net import URI
from java.net import URISyntaxException
from java.net import URL
from java.nio.file import FileStore
from java.nio.file import FileSystems
from java.nio.file import Path
from java.security import SecureRandom
from sun.security.action import GetPropertyAction
from typing import Any, Callable, Iterable, Tuple


class File(Serializable, Comparable):

    separatorChar = fs.getSeparator()
    """
    The system-dependent default name-separator character.  This field is
    initialized to contain the first character of the value of the system
    property `file.separator`.  On UNIX systems the value of this
    field is `'/'`; on Microsoft Windows systems it is `'\\'`.

    See
    - java.lang.System.getProperty(java.lang.String)
    """
    separator = "" + separatorChar
    """
    The system-dependent default name-separator character, represented as a
    string for convenience.  This string contains a single character, namely
    .separatorChar.
    """
    pathSeparatorChar = fs.getPathSeparator()
    """
    The system-dependent path-separator character.  This field is
    initialized to contain the first character of the value of the system
    property `path.separator`.  This character is used to
    separate filenames in a sequence of files given as a *path list*.
    On UNIX systems, this character is `':'`; on Microsoft Windows systems it
    is `';'`.

    See
    - java.lang.System.getProperty(java.lang.String)
    """
    pathSeparator = "" + pathSeparatorChar
    """
    The system-dependent path-separator character, represented as a string
    for convenience.  This string contains a single character, namely
    .pathSeparatorChar.
    """


    def __init__(self, pathname: str):
        """
        Creates a new `File` instance by converting the given
        pathname string into an abstract pathname.  If the given string is
        the empty string, then the result is the empty abstract pathname.

        Arguments
        - pathname: A pathname string

        Raises
        - NullPointerException: If the `pathname` argument is `null`
        """
        ...


    def __init__(self, parent: str, child: str):
        """
        Creates a new `File` instance from a parent pathname string
        and a child pathname string.
        
         If `parent` is `null` then the new
        `File` instance is created as if by invoking the
        single-argument `File` constructor on the given
        `child` pathname string.
        
         Otherwise the `parent` pathname string is taken to denote
        a directory, and the `child` pathname string is taken to
        denote either a directory or a file.  If the `child` pathname
        string is absolute then it is converted into a relative pathname in a
        system-dependent way.  If `parent` is the empty string then
        the new `File` instance is created by converting
        `child` into an abstract pathname and resolving the result
        against a system-dependent default directory.  Otherwise each pathname
        string is converted into an abstract pathname and the child abstract
        pathname is resolved against the parent.

        Arguments
        - parent: The parent pathname string
        - child: The child pathname string

        Raises
        - NullPointerException: If `child` is `null`
        """
        ...


    def __init__(self, parent: "File", child: str):
        """
        Creates a new `File` instance from a parent abstract
        pathname and a child pathname string.
        
         If `parent` is `null` then the new
        `File` instance is created as if by invoking the
        single-argument `File` constructor on the given
        `child` pathname string.
        
         Otherwise the `parent` abstract pathname is taken to
        denote a directory, and the `child` pathname string is taken
        to denote either a directory or a file.  If the `child`
        pathname string is absolute then it is converted into a relative
        pathname in a system-dependent way.  If `parent` is the empty
        abstract pathname then the new `File` instance is created by
        converting `child` into an abstract pathname and resolving
        the result against a system-dependent default directory.  Otherwise each
        pathname string is converted into an abstract pathname and the child
        abstract pathname is resolved against the parent.

        Arguments
        - parent: The parent abstract pathname
        - child: The child pathname string

        Raises
        - NullPointerException: If `child` is `null`
        """
        ...


    def __init__(self, uri: "URI"):
        """
        Creates a new `File` instance by converting the given
        `file:` URI into an abstract pathname.
        
         The exact form of a `file:` URI is system-dependent, hence
        the transformation performed by this constructor is also
        system-dependent.
        
         For a given abstract pathname *f* it is guaranteed that
        
        <blockquote>`
        new File(`*&nbsp;f*`..toURI()
        toURI()).equals(`*&nbsp;f*`..getAbsoluteFile() getAbsoluteFile())
        `</blockquote>
        
        so long as the original abstract pathname, the URI, and the new abstract
        pathname are all created in (possibly different invocations of) the same
        Java virtual machine.  This relationship typically does not hold,
        however, when a `file:` URI that is created in a virtual machine
        on one operating system is converted into an abstract pathname in a
        virtual machine on a different operating system.

        Arguments
        - uri: An absolute, hierarchical URI with a scheme equal to
                `"file"`, a non-empty path component, and undefined
                authority, query, and fragment components

        Raises
        - NullPointerException: If `uri` is `null`
        - IllegalArgumentException: If the preconditions on the parameter do not hold

        See
        - java.net.URI

        Since
        - 1.4
        """
        ...


    def getName(self) -> str:
        """
        Returns the name of the file or directory denoted by this abstract
        pathname.  This is just the last name in the pathname's name
        sequence.  If the pathname's name sequence is empty, then the empty
        string is returned.

        Returns
        - The name of the file or directory denoted by this abstract
                 pathname, or the empty string if this pathname's name sequence
                 is empty
        """
        ...


    def getParent(self) -> str:
        """
        Returns the pathname string of this abstract pathname's parent, or
        `null` if this pathname does not name a parent directory.
        
         The *parent* of an abstract pathname consists of the
        pathname's prefix, if any, and each name in the pathname's name
        sequence except for the last.  If the name sequence is empty then
        the pathname does not name a parent directory.

        Returns
        - The pathname string of the parent directory named by this
                 abstract pathname, or `null` if this pathname
                 does not name a parent
        """
        ...


    def getParentFile(self) -> "File":
        """
        Returns the abstract pathname of this abstract pathname's parent,
        or `null` if this pathname does not name a parent
        directory.
        
         The *parent* of an abstract pathname consists of the
        pathname's prefix, if any, and each name in the pathname's name
        sequence except for the last.  If the name sequence is empty then
        the pathname does not name a parent directory.

        Returns
        - The abstract pathname of the parent directory named by this
                 abstract pathname, or `null` if this pathname
                 does not name a parent

        Since
        - 1.2
        """
        ...


    def getPath(self) -> str:
        """
        Converts this abstract pathname into a pathname string.  The resulting
        string uses the .separator default name-separator character to
        separate the names in the name sequence.

        Returns
        - The string form of this abstract pathname
        """
        ...


    def isAbsolute(self) -> bool:
        """
        Tests whether this abstract pathname is absolute.  The definition of
        absolute pathname is system dependent.  On UNIX systems, a pathname is
        absolute if its prefix is `"/"`.  On Microsoft Windows systems, a
        pathname is absolute if its prefix is a drive specifier followed by
        `"\\"`, or if its prefix is `"\\\\"`.

        Returns
        - `True` if this abstract pathname is absolute,
                 `False` otherwise
        """
        ...


    def getAbsolutePath(self) -> str:
        """
        Returns the absolute pathname string of this abstract pathname.
        
         If this abstract pathname is already absolute, then the pathname
        string is simply returned as if by the .getPath
        method.  If this abstract pathname is the empty abstract pathname then
        the pathname string of the current user directory, which is named by the
        system property `user.dir`, is returned.  Otherwise this
        pathname is resolved in a system-dependent way.  On UNIX systems, a
        relative pathname is made absolute by resolving it against the current
        user directory.  On Microsoft Windows systems, a relative pathname is made absolute
        by resolving it against the current directory of the drive named by the
        pathname, if any; if not, it is resolved against the current user
        directory.

        Returns
        - The absolute pathname string denoting the same file or
                 directory as this abstract pathname

        Raises
        - SecurityException: If a required system property value cannot be accessed.

        See
        - java.io.File.isAbsolute()
        """
        ...


    def getAbsoluteFile(self) -> "File":
        """
        Returns the absolute form of this abstract pathname.  Equivalent to
        `new&nbsp;File(this..getAbsolutePath)`.

        Returns
        - The absolute abstract pathname denoting the same file or
                 directory as this abstract pathname

        Raises
        - SecurityException: If a required system property value cannot be accessed.

        Since
        - 1.2
        """
        ...


    def getCanonicalPath(self) -> str:
        """
        Returns the canonical pathname string of this abstract pathname.
        
         A canonical pathname is both absolute and unique.  The precise
        definition of canonical form is system-dependent.  This method first
        converts this pathname to absolute form if necessary, as if by invoking the
        .getAbsolutePath method, and then maps it to its unique form in a
        system-dependent way.  This typically involves removing redundant names
        such as `"."` and `".."` from the pathname, resolving
        symbolic links (on UNIX platforms), and converting drive letters to a
        standard case (on Microsoft Windows platforms).
        
         Every pathname that denotes an existing file or directory has a
        unique canonical form.  Every pathname that denotes a nonexistent file
        or directory also has a unique canonical form.  The canonical form of
        the pathname of a nonexistent file or directory may be different from
        the canonical form of the same pathname after the file or directory is
        created.  Similarly, the canonical form of the pathname of an existing
        file or directory may be different from the canonical form of the same
        pathname after the file or directory is deleted.

        Returns
        - The canonical pathname string denoting the same file or
                 directory as this abstract pathname

        Raises
        - IOException: If an I/O error occurs, which is possible because the
                 construction of the canonical pathname may require
                 filesystem queries
        - SecurityException: If a required system property value cannot be accessed, or
                 if a security manager exists and its java.lang.SecurityManager.checkRead method denies
                 read access to the file

        See
        - Path.toRealPath

        Since
        - 1.1
        """
        ...


    def getCanonicalFile(self) -> "File":
        """
        Returns the canonical form of this abstract pathname.  Equivalent to
        `new&nbsp;File(this..getCanonicalPath)`.

        Returns
        - The canonical pathname string denoting the same file or
                 directory as this abstract pathname

        Raises
        - IOException: If an I/O error occurs, which is possible because the
                 construction of the canonical pathname may require
                 filesystem queries
        - SecurityException: If a required system property value cannot be accessed, or
                 if a security manager exists and its java.lang.SecurityManager.checkRead method denies
                 read access to the file

        See
        - Path.toRealPath

        Since
        - 1.2
        """
        ...


    def toURL(self) -> "URL":
        """
        Converts this abstract pathname into a `file:` URL.  The
        exact form of the URL is system-dependent.  If it can be determined that
        the file denoted by this abstract pathname is a directory, then the
        resulting URL will end with a slash.

        Returns
        - A URL object representing the equivalent file URL

        Raises
        - MalformedURLException: If the path cannot be parsed as a URL

        See
        - java.net.URL

        Since
        - 1.2

        Deprecated
        - This method does not automatically escape characters that
        are illegal in URLs.  It is recommended that new code convert an
        abstract pathname into a URL by first converting it into a URI, via the
        .toURI() toURI method, and then converting the URI into a URL
        via the java.net.URI.toURL() URI.toURL method.
        """
        ...


    def toURI(self) -> "URI":
        """
        Constructs a `file:` URI that represents this abstract pathname.
        
         The exact form of the URI is system-dependent.  If it can be
        determined that the file denoted by this abstract pathname is a
        directory, then the resulting URI will end with a slash.
        
         For a given abstract pathname *f*, it is guaranteed that
        
        <blockquote>`
        new .File(java.net.URI) File(`*&nbsp;f*`.toURI()).equals(
        `*&nbsp;f*`..getAbsoluteFile() getAbsoluteFile())
        `</blockquote>
        
        so long as the original abstract pathname, the URI, and the new abstract
        pathname are all created in (possibly different invocations of) the same
        Java virtual machine.  Due to the system-dependent nature of abstract
        pathnames, however, this relationship typically does not hold when a
        `file:` URI that is created in a virtual machine on one operating
        system is converted into an abstract pathname in a virtual machine on a
        different operating system.
        
         Note that when this abstract pathname represents a UNC pathname then
        all components of the UNC (including the server name component) are encoded
        in the `URI` path. The authority component is undefined, meaning
        that it is represented as `null`. The Path class defines the
        Path.toUri toUri method to encode the server name in the authority
        component of the resulting `URI`. The .toPath toPath method
        may be used to obtain a `Path` representing this abstract pathname.

        Returns
        - An absolute, hierarchical URI with a scheme equal to
                 `"file"`, a path representing this abstract pathname,
                 and undefined authority, query, and fragment components

        Raises
        - SecurityException: If a required system property value cannot
        be accessed.

        See
        - java.net.URI.toURL()

        Since
        - 1.4
        """
        ...


    def canRead(self) -> bool:
        """
        Tests whether the application can read the file denoted by this
        abstract pathname. On some platforms it may be possible to start the
        Java virtual machine with special privileges that allow it to read
        files that are marked as unreadable. Consequently this method may return
        `True` even though the file does not have read permissions.

        Returns
        - `True` if and only if the file specified by this
                 abstract pathname exists *and* can be read by the
                 application; `False` otherwise

        Raises
        - SecurityException: If a security manager exists and its java.lang.SecurityManager.checkRead(java.lang.String)
                 method denies read access to the file
        """
        ...


    def canWrite(self) -> bool:
        """
        Tests whether the application can modify the file denoted by this
        abstract pathname. On some platforms it may be possible to start the
        Java virtual machine with special privileges that allow it to modify
        files that are marked read-only. Consequently this method may return
        `True` even though the file is marked read-only.

        Returns
        - `True` if and only if the file system actually
                 contains a file denoted by this abstract pathname *and*
                 the application is allowed to write to the file;
                 `False` otherwise.

        Raises
        - SecurityException: If a security manager exists and its java.lang.SecurityManager.checkWrite(java.lang.String)
                 method denies write access to the file
        """
        ...


    def exists(self) -> bool:
        """
        Tests whether the file or directory denoted by this abstract pathname
        exists.

        Returns
        - `True` if and only if the file or directory denoted
                 by this abstract pathname exists; `False` otherwise

        Raises
        - SecurityException: If a security manager exists and its java.lang.SecurityManager.checkRead(java.lang.String)
                 method denies read access to the file or directory
        """
        ...


    def isDirectory(self) -> bool:
        """
        Tests whether the file denoted by this abstract pathname is a
        directory.
        
         Where it is required to distinguish an I/O exception from the case
        that the file is not a directory, or where several attributes of the
        same file are required at the same time, then the java.nio.file.Files.readAttributes(Path,Class,LinkOption[])
        Files.readAttributes method may be used.

        Returns
        - `True` if and only if the file denoted by this
                 abstract pathname exists *and* is a directory;
                 `False` otherwise

        Raises
        - SecurityException: If a security manager exists and its java.lang.SecurityManager.checkRead(java.lang.String)
                 method denies read access to the file
        """
        ...


    def isFile(self) -> bool:
        """
        Tests whether the file denoted by this abstract pathname is a normal
        file.  A file is *normal* if it is not a directory and, in
        addition, satisfies other system-dependent criteria.  Any non-directory
        file created by a Java application is guaranteed to be a normal file.
        
         Where it is required to distinguish an I/O exception from the case
        that the file is not a normal file, or where several attributes of the
        same file are required at the same time, then the java.nio.file.Files.readAttributes(Path,Class,LinkOption[])
        Files.readAttributes method may be used.

        Returns
        - `True` if and only if the file denoted by this
                 abstract pathname exists *and* is a normal file;
                 `False` otherwise

        Raises
        - SecurityException: If a security manager exists and its java.lang.SecurityManager.checkRead(java.lang.String)
                 method denies read access to the file
        """
        ...


    def isHidden(self) -> bool:
        """
        Tests whether the file named by this abstract pathname is a hidden
        file.  The exact definition of *hidden* is system-dependent.  On
        UNIX systems, a file is considered to be hidden if its name begins with
        a period character (`'.'`).  On Microsoft Windows systems, a file is
        considered to be hidden if it has been marked as such in the filesystem.

        Returns
        - `True` if and only if the file denoted by this
                 abstract pathname is hidden according to the conventions of the
                 underlying platform

        Raises
        - SecurityException: If a security manager exists and its java.lang.SecurityManager.checkRead(java.lang.String)
                 method denies read access to the file

        Since
        - 1.2
        """
        ...


    def lastModified(self) -> int:
        """
        Returns the time that the file denoted by this abstract pathname was
        last modified.

        Returns
        - A `long` value representing the time the file was
                 last modified, measured in milliseconds since the epoch
                 (00:00:00 GMT, January 1, 1970), or `0L` if the
                 file does not exist or if an I/O error occurs.  The value may
                 be negative indicating the number of milliseconds before the
                 epoch

        Raises
        - SecurityException: If a security manager exists and its java.lang.SecurityManager.checkRead(java.lang.String)
                 method denies read access to the file

        Unknown Tags
        - While the unit of time of the return value is milliseconds, the
        granularity of the value depends on the underlying file system and may
        be larger.  For example, some file systems use time stamps in units of
        seconds.
        
         Where it is required to distinguish an I/O exception from the case
        where `0L` is returned, or where several attributes of the
        same file are required at the same time, or where the time of last
        access or the creation time are required, then the java.nio.file.Files.readAttributes(Path,Class,LinkOption[])
        Files.readAttributes method may be used.  If however only the
        time of last modification is required, then the
        java.nio.file.Files.getLastModifiedTime(Path,LinkOption[])
        Files.getLastModifiedTime method may be used instead.
        """
        ...


    def length(self) -> int:
        """
        Returns the length of the file denoted by this abstract pathname.
        The return value is unspecified if this pathname denotes a directory.
        
         Where it is required to distinguish an I/O exception from the case
        that `0L` is returned, or where several attributes of the same file
        are required at the same time, then the java.nio.file.Files.readAttributes(Path,Class,LinkOption[])
        Files.readAttributes method may be used.

        Returns
        - The length, in bytes, of the file denoted by this abstract
                 pathname, or `0L` if the file does not exist.  Some
                 operating systems may return `0L` for pathnames
                 denoting system-dependent entities such as devices or pipes.

        Raises
        - SecurityException: If a security manager exists and its java.lang.SecurityManager.checkRead(java.lang.String)
                 method denies read access to the file
        """
        ...


    def createNewFile(self) -> bool:
        """
        Atomically creates a new, empty file named by this abstract pathname if
        and only if a file with this name does not yet exist.  The check for the
        existence of the file and the creation of the file if it does not exist
        are a single operation that is atomic with respect to all other
        filesystem activities that might affect the file.
        <P>
        Note: this method should *not* be used for file-locking, as
        the resulting protocol cannot be made to work reliably. The
        java.nio.channels.FileLock FileLock
        facility should be used instead.

        Returns
        - `True` if the named file does not exist and was
                 successfully created; `False` if the named file
                 already exists

        Raises
        - IOException: If an I/O error occurred
        - SecurityException: If a security manager exists and its java.lang.SecurityManager.checkWrite(java.lang.String)
                 method denies write access to the file

        Since
        - 1.2
        """
        ...


    def delete(self) -> bool:
        """
        Deletes the file or directory denoted by this abstract pathname.  If
        this pathname denotes a directory, then the directory must be empty in
        order to be deleted.
        
         Note that the java.nio.file.Files class defines the java.nio.file.Files.delete(Path) delete method to throw an IOException
        when a file cannot be deleted. This is useful for error reporting and to
        diagnose why a file cannot be deleted.

        Returns
        - `True` if and only if the file or directory is
                 successfully deleted; `False` otherwise

        Raises
        - SecurityException: If a security manager exists and its java.lang.SecurityManager.checkDelete method denies
                 delete access to the file
        """
        ...


    def deleteOnExit(self) -> None:
        """
        Requests that the file or directory denoted by this abstract
        pathname be deleted when the virtual machine terminates.
        Files (or directories) are deleted in the reverse order that
        they are registered. Invoking this method to delete a file or
        directory that is already registered for deletion has no effect.
        Deletion will be attempted only for normal termination of the
        virtual machine, as defined by the Java Language Specification.
        
         Once deletion has been requested, it is not possible to cancel the
        request.  This method should therefore be used with care.
        
        <P>
        Note: this method should *not* be used for file-locking, as
        the resulting protocol cannot be made to work reliably. The
        java.nio.channels.FileLock FileLock
        facility should be used instead.

        Raises
        - SecurityException: If a security manager exists and its java.lang.SecurityManager.checkDelete method denies
                 delete access to the file

        See
        - .delete

        Since
        - 1.2
        """
        ...


    def list(self) -> list[str]:
        """
        Returns an array of strings naming the files and directories in the
        directory denoted by this abstract pathname.
        
         If this abstract pathname does not denote a directory, then this
        method returns `null`.  Otherwise an array of strings is
        returned, one for each file or directory in the directory.  Names
        denoting the directory itself and the directory's parent directory are
        not included in the result.  Each string is a file name rather than a
        complete path.
        
         There is no guarantee that the name strings in the resulting array
        will appear in any specific order; they are not, in particular,
        guaranteed to appear in alphabetical order.
        
         Note that the java.nio.file.Files class defines the java.nio.file.Files.newDirectoryStream(Path) newDirectoryStream method to
        open a directory and iterate over the names of the files in the directory.
        This may use less resources when working with very large directories, and
        may be more responsive when working with remote directories.

        Returns
        - An array of strings naming the files and directories in the
                 directory denoted by this abstract pathname.  The array will be
                 empty if the directory is empty.  Returns `null` if
                 this abstract pathname does not denote a directory, or if an
                 I/O error occurs.

        Raises
        - SecurityException: If a security manager exists and its SecurityManager.checkRead(String) method denies read access to
                 the directory
        """
        ...


    def list(self, filter: "FilenameFilter") -> list[str]:
        """
        Returns an array of strings naming the files and directories in the
        directory denoted by this abstract pathname that satisfy the specified
        filter.  The behavior of this method is the same as that of the
        .list() method, except that the strings in the returned array
        must satisfy the filter.  If the given `filter` is `null`
        then all names are accepted.  Otherwise, a name satisfies the filter if
        and only if the value `True` results when the FilenameFilter.accept FilenameFilter.accept(File,&nbsp;String) method
        of the filter is invoked on this abstract pathname and the name of a
        file or directory in the directory that it denotes.

        Arguments
        - filter: A filename filter

        Returns
        - An array of strings naming the files and directories in the
                 directory denoted by this abstract pathname that were accepted
                 by the given `filter`.  The array will be empty if the
                 directory is empty or if no names were accepted by the filter.
                 Returns `null` if this abstract pathname does not denote
                 a directory, or if an I/O error occurs.

        Raises
        - SecurityException: If a security manager exists and its SecurityManager.checkRead(String) method denies read access to
                 the directory

        See
        - java.nio.file.Files.newDirectoryStream(Path,String)
        """
        ...


    def listFiles(self) -> list["File"]:
        """
        Returns an array of abstract pathnames denoting the files in the
        directory denoted by this abstract pathname.
        
         If this abstract pathname does not denote a directory, then this
        method returns `null`.  Otherwise an array of `File` objects
        is returned, one for each file or directory in the directory.  Pathnames
        denoting the directory itself and the directory's parent directory are
        not included in the result.  Each resulting abstract pathname is
        constructed from this abstract pathname using the .File(File,
        String) File(File,&nbsp;String) constructor.  Therefore if this
        pathname is absolute then each resulting pathname is absolute; if this
        pathname is relative then each resulting pathname will be relative to
        the same directory.
        
         There is no guarantee that the name strings in the resulting array
        will appear in any specific order; they are not, in particular,
        guaranteed to appear in alphabetical order.
        
         Note that the java.nio.file.Files class defines the java.nio.file.Files.newDirectoryStream(Path) newDirectoryStream method
        to open a directory and iterate over the names of the files in the
        directory. This may use less resources when working with very large
        directories.

        Returns
        - An array of abstract pathnames denoting the files and
                 directories in the directory denoted by this abstract pathname.
                 The array will be empty if the directory is empty.  Returns
                 `null` if this abstract pathname does not denote a
                 directory, or if an I/O error occurs.

        Raises
        - SecurityException: If a security manager exists and its SecurityManager.checkRead(String) method denies read access to
                 the directory

        Since
        - 1.2
        """
        ...


    def listFiles(self, filter: "FilenameFilter") -> list["File"]:
        """
        Returns an array of abstract pathnames denoting the files and
        directories in the directory denoted by this abstract pathname that
        satisfy the specified filter.  The behavior of this method is the same
        as that of the .listFiles() method, except that the pathnames in
        the returned array must satisfy the filter.  If the given `filter`
        is `null` then all pathnames are accepted.  Otherwise, a pathname
        satisfies the filter if and only if the value `True` results when
        the FilenameFilter.accept
        FilenameFilter.accept(File,&nbsp;String) method of the filter is
        invoked on this abstract pathname and the name of a file or directory in
        the directory that it denotes.

        Arguments
        - filter: A filename filter

        Returns
        - An array of abstract pathnames denoting the files and
                 directories in the directory denoted by this abstract pathname.
                 The array will be empty if the directory is empty.  Returns
                 `null` if this abstract pathname does not denote a
                 directory, or if an I/O error occurs.

        Raises
        - SecurityException: If a security manager exists and its SecurityManager.checkRead(String) method denies read access to
                 the directory

        See
        - java.nio.file.Files.newDirectoryStream(Path,String)

        Since
        - 1.2
        """
        ...


    def listFiles(self, filter: "FileFilter") -> list["File"]:
        """
        Returns an array of abstract pathnames denoting the files and
        directories in the directory denoted by this abstract pathname that
        satisfy the specified filter.  The behavior of this method is the same
        as that of the .listFiles() method, except that the pathnames in
        the returned array must satisfy the filter.  If the given `filter`
        is `null` then all pathnames are accepted.  Otherwise, a pathname
        satisfies the filter if and only if the value `True` results when
        the FileFilter.accept FileFilter.accept(File) method of the
        filter is invoked on the pathname.

        Arguments
        - filter: A file filter

        Returns
        - An array of abstract pathnames denoting the files and
                 directories in the directory denoted by this abstract pathname.
                 The array will be empty if the directory is empty.  Returns
                 `null` if this abstract pathname does not denote a
                 directory, or if an I/O error occurs.

        Raises
        - SecurityException: If a security manager exists and its SecurityManager.checkRead(String) method denies read access to
                 the directory

        See
        - java.nio.file.Files.newDirectoryStream(Path,java.nio.file.DirectoryStream.Filter)

        Since
        - 1.2
        """
        ...


    def mkdir(self) -> bool:
        """
        Creates the directory named by this abstract pathname.

        Returns
        - `True` if and only if the directory was
                 created; `False` otherwise

        Raises
        - SecurityException: If a security manager exists and its java.lang.SecurityManager.checkWrite(java.lang.String)
                 method does not permit the named directory to be created
        """
        ...


    def mkdirs(self) -> bool:
        """
        Creates the directory named by this abstract pathname, including any
        necessary but nonexistent parent directories.  Note that if this
        operation fails it may have succeeded in creating some of the necessary
        parent directories.

        Returns
        - `True` if and only if the directory was created,
                 along with all necessary parent directories; `False`
                 otherwise

        Raises
        - SecurityException: If a security manager exists and its java.lang.SecurityManager.checkRead(java.lang.String)
                 method does not permit verification of the existence of the
                 named directory and all necessary parent directories; or if
                 the java.lang.SecurityManager.checkWrite(java.lang.String)
                 method does not permit the named directory and all necessary
                 parent directories to be created
        """
        ...


    def renameTo(self, dest: "File") -> bool:
        """
        Renames the file denoted by this abstract pathname.
        
         Many aspects of the behavior of this method are inherently
        platform-dependent: The rename operation might not be able to move a
        file from one filesystem to another, it might not be atomic, and it
        might not succeed if a file with the destination abstract pathname
        already exists.  The return value should always be checked to make sure
        that the rename operation was successful.  As instances of `File`
        are immutable, this File object is not changed to name the destination
        file or directory.
        
         Note that the java.nio.file.Files class defines the java.nio.file.Files.move move method to move or rename a file in a
        platform independent manner.

        Arguments
        - dest: The new abstract pathname for the named file

        Returns
        - `True` if and only if the renaming succeeded;
                 `False` otherwise

        Raises
        - SecurityException: If a security manager exists and its java.lang.SecurityManager.checkWrite(java.lang.String)
                 method denies write access to either the old or new pathnames
        - NullPointerException: If parameter `dest` is `null`
        """
        ...


    def setLastModified(self, time: int) -> bool:
        """
        Sets the last-modified time of the file or directory named by this
        abstract pathname.
        
         All platforms support file-modification times to the nearest second,
        but some provide more precision.  The argument will be truncated to fit
        the supported precision.  If the operation succeeds and no intervening
        operations on the file take place, then the next invocation of the
        .lastModified method will return the (possibly
        truncated) `time` argument that was passed to this method.

        Arguments
        - time: The new last-modified time, measured in milliseconds since
                      the epoch (00:00:00 GMT, January 1, 1970)

        Returns
        - `True` if and only if the operation succeeded;
                 `False` otherwise

        Raises
        - IllegalArgumentException: If the argument is negative
        - SecurityException: If a security manager exists and its java.lang.SecurityManager.checkWrite(java.lang.String)
                 method denies write access to the named file

        Since
        - 1.2
        """
        ...


    def setReadOnly(self) -> bool:
        """
        Marks the file or directory named by this abstract pathname so that
        only read operations are allowed. After invoking this method the file
        or directory will not change until it is either deleted or marked
        to allow write access. On some platforms it may be possible to start the
        Java virtual machine with special privileges that allow it to modify
        files that are marked read-only. Whether or not a read-only file or
        directory may be deleted depends upon the underlying system.

        Returns
        - `True` if and only if the operation succeeded;
                 `False` otherwise

        Raises
        - SecurityException: If a security manager exists and its java.lang.SecurityManager.checkWrite(java.lang.String)
                 method denies write access to the named file

        Since
        - 1.2
        """
        ...


    def setWritable(self, writable: bool, ownerOnly: bool) -> bool:
        """
        Sets the owner's or everybody's write permission for this abstract
        pathname. On some platforms it may be possible to start the Java virtual
        machine with special privileges that allow it to modify files that
        disallow write operations.
        
         The java.nio.file.Files class defines methods that operate on
        file attributes including file permissions. This may be used when finer
        manipulation of file permissions is required.

        Arguments
        - writable: If `True`, sets the access permission to allow write
                 operations; if `False` to disallow write operations
        - ownerOnly: If `True`, the write permission applies only to the
                 owner's write permission; otherwise, it applies to everybody.  If
                 the underlying file system can not distinguish the owner's write
                 permission from that of others, then the permission will apply to
                 everybody, regardless of this value.

        Returns
        - `True` if and only if the operation succeeded. The
                 operation will fail if the user does not have permission to change
                 the access permissions of this abstract pathname.

        Raises
        - SecurityException: If a security manager exists and its java.lang.SecurityManager.checkWrite(java.lang.String)
                 method denies write access to the named file

        Since
        - 1.6
        """
        ...


    def setWritable(self, writable: bool) -> bool:
        """
        A convenience method to set the owner's write permission for this abstract
        pathname. On some platforms it may be possible to start the Java virtual
        machine with special privileges that allow it to modify files that
        disallow write operations.
        
         An invocation of this method of the form `file.setWritable(arg)`
        behaves in exactly the same way as the invocation
        
        ````file.setWritable(arg, True)````

        Arguments
        - writable: If `True`, sets the access permission to allow write
                 operations; if `False` to disallow write operations

        Returns
        - `True` if and only if the operation succeeded.  The
                 operation will fail if the user does not have permission to
                 change the access permissions of this abstract pathname.

        Raises
        - SecurityException: If a security manager exists and its java.lang.SecurityManager.checkWrite(java.lang.String)
                 method denies write access to the file

        Since
        - 1.6
        """
        ...


    def setReadable(self, readable: bool, ownerOnly: bool) -> bool:
        """
        Sets the owner's or everybody's read permission for this abstract
        pathname. On some platforms it may be possible to start the Java virtual
        machine with special privileges that allow it to read files that are
        marked as unreadable.
        
         The java.nio.file.Files class defines methods that operate on
        file attributes including file permissions. This may be used when finer
        manipulation of file permissions is required.

        Arguments
        - readable: If `True`, sets the access permission to allow read
                 operations; if `False` to disallow read operations
        - ownerOnly: If `True`, the read permission applies only to the
                 owner's read permission; otherwise, it applies to everybody.  If
                 the underlying file system can not distinguish the owner's read
                 permission from that of others, then the permission will apply to
                 everybody, regardless of this value.

        Returns
        - `True` if and only if the operation succeeded.  The
                 operation will fail if the user does not have permission to
                 change the access permissions of this abstract pathname.  If
                 `readable` is `False` and the underlying
                 file system does not implement a read permission, then the
                 operation will fail.

        Raises
        - SecurityException: If a security manager exists and its java.lang.SecurityManager.checkWrite(java.lang.String)
                 method denies write access to the file

        Since
        - 1.6
        """
        ...


    def setReadable(self, readable: bool) -> bool:
        """
        A convenience method to set the owner's read permission for this abstract
        pathname. On some platforms it may be possible to start the Java virtual
        machine with special privileges that allow it to read files that are
        marked as unreadable.
        
        An invocation of this method of the form `file.setReadable(arg)`
        behaves in exactly the same way as the invocation
        
        ````file.setReadable(arg, True)````

        Arguments
        - readable: If `True`, sets the access permission to allow read
                 operations; if `False` to disallow read operations

        Returns
        - `True` if and only if the operation succeeded.  The
                 operation will fail if the user does not have permission to
                 change the access permissions of this abstract pathname.  If
                 `readable` is `False` and the underlying
                 file system does not implement a read permission, then the
                 operation will fail.

        Raises
        - SecurityException: If a security manager exists and its java.lang.SecurityManager.checkWrite(java.lang.String)
                 method denies write access to the file

        Since
        - 1.6
        """
        ...


    def setExecutable(self, executable: bool, ownerOnly: bool) -> bool:
        """
        Sets the owner's or everybody's execute permission for this abstract
        pathname. On some platforms it may be possible to start the Java virtual
        machine with special privileges that allow it to execute files that are
        not marked executable.
        
         The java.nio.file.Files class defines methods that operate on
        file attributes including file permissions. This may be used when finer
        manipulation of file permissions is required.

        Arguments
        - executable: If `True`, sets the access permission to allow execute
                 operations; if `False` to disallow execute operations
        - ownerOnly: If `True`, the execute permission applies only to the
                 owner's execute permission; otherwise, it applies to everybody.
                 If the underlying file system can not distinguish the owner's
                 execute permission from that of others, then the permission will
                 apply to everybody, regardless of this value.

        Returns
        - `True` if and only if the operation succeeded.  The
                 operation will fail if the user does not have permission to
                 change the access permissions of this abstract pathname.  If
                 `executable` is `False` and the underlying
                 file system does not implement an execute permission, then the
                 operation will fail.

        Raises
        - SecurityException: If a security manager exists and its java.lang.SecurityManager.checkWrite(java.lang.String)
                 method denies write access to the file

        Since
        - 1.6
        """
        ...


    def setExecutable(self, executable: bool) -> bool:
        """
        A convenience method to set the owner's execute permission for this
        abstract pathname. On some platforms it may be possible to start the Java
        virtual machine with special privileges that allow it to execute files
        that are not marked executable.
        
        An invocation of this method of the form `file.setExcutable(arg)`
        behaves in exactly the same way as the invocation
        
        ````file.setExecutable(arg, True)````

        Arguments
        - executable: If `True`, sets the access permission to allow execute
                 operations; if `False` to disallow execute operations

        Returns
        - `True` if and only if the operation succeeded.  The
                  operation will fail if the user does not have permission to
                  change the access permissions of this abstract pathname.  If
                  `executable` is `False` and the underlying
                  file system does not implement an execute permission, then the
                  operation will fail.

        Raises
        - SecurityException: If a security manager exists and its java.lang.SecurityManager.checkWrite(java.lang.String)
                 method denies write access to the file

        Since
        - 1.6
        """
        ...


    def canExecute(self) -> bool:
        """
        Tests whether the application can execute the file denoted by this
        abstract pathname. On some platforms it may be possible to start the
        Java virtual machine with special privileges that allow it to execute
        files that are not marked executable. Consequently this method may return
        `True` even though the file does not have execute permissions.

        Returns
        - `True` if and only if the abstract pathname exists
                 *and* the application is allowed to execute the file

        Raises
        - SecurityException: If a security manager exists and its java.lang.SecurityManager.checkExec(java.lang.String)
                 method denies execute access to the file

        Since
        - 1.6
        """
        ...


    @staticmethod
    def listRoots() -> list["File"]:
        """
        List the available filesystem roots.
        
         A particular Java platform may support zero or more
        hierarchically-organized file systems.  Each file system has a
        `root` directory from which all other files in that file system
        can be reached.  Windows platforms, for example, have a root directory
        for each active drive; UNIX platforms have a single root directory,
        namely `"/"`.  The set of available filesystem roots is affected
        by various system-level operations such as the insertion or ejection of
        removable media and the disconnecting or unmounting of physical or
        virtual disk drives.
        
         This method returns an array of `File` objects that denote the
        root directories of the available filesystem roots.  It is guaranteed
        that the canonical pathname of any file physically present on the local
        machine will begin with one of the roots returned by this method.
        
         The canonical pathname of a file that resides on some other machine
        and is accessed via a remote-filesystem protocol such as SMB or NFS may
        or may not begin with one of the roots returned by this method.  If the
        pathname of a remote file is syntactically indistinguishable from the
        pathname of a local file then it will begin with one of the roots
        returned by this method.  Thus, for example, `File` objects
        denoting the root directories of the mapped network drives of a Windows
        platform will be returned by this method, while `File` objects
        containing UNC pathnames will not be returned by this method.
        
         Unlike most methods in this class, this method does not throw
        security exceptions.  If a security manager exists and its SecurityManager.checkRead(String) method denies read access to a
        particular root directory, then that directory will not appear in the
        result.

        Returns
        - An array of `File` objects denoting the available
                 filesystem roots, or `null` if the set of roots could not
                 be determined.  The array will be empty if there are no
                 filesystem roots.

        See
        - java.nio.file.FileStore

        Since
        - 1.2
        """
        ...


    def getTotalSpace(self) -> int:
        """
        Returns the size of the partition <a href="#partName">named</a> by this
        abstract pathname. If the total number of bytes in the partition is
        greater than Long.MAX_VALUE, then `Long.MAX_VALUE` will be
        returned.

        Returns
        - The size, in bytes, of the partition or `0L` if this
                 abstract pathname does not name a partition or if the size
                 cannot be obtained

        Raises
        - SecurityException: If a security manager has been installed and it denies
                 RuntimePermission`("getFileSystemAttributes")`
                 or its SecurityManager.checkRead(String) method denies
                 read access to the file named by this abstract pathname

        See
        - FileStore.getTotalSpace

        Since
        - 1.6
        """
        ...


    def getFreeSpace(self) -> int:
        """
        Returns the number of unallocated bytes in the partition <a
        href="#partName">named</a> by this abstract path name.  If the
        number of unallocated bytes in the partition is greater than
        Long.MAX_VALUE, then `Long.MAX_VALUE` will be returned.
        
         The returned number of unallocated bytes is a hint, but not
        a guarantee, that it is possible to use most or any of these
        bytes.  The number of unallocated bytes is most likely to be
        accurate immediately after this call.  It is likely to be made
        inaccurate by any external I/O operations including those made
        on the system outside of this virtual machine.  This method
        makes no guarantee that write operations to this file system
        will succeed.

        Returns
        - The number of unallocated bytes on the partition or `0L`
                 if the abstract pathname does not name a partition or if this
                 number cannot be obtained.  This value will be less than or
                 equal to the total file system size returned by
                 .getTotalSpace.

        Raises
        - SecurityException: If a security manager has been installed and it denies
                 RuntimePermission`("getFileSystemAttributes")`
                 or its SecurityManager.checkRead(String) method denies
                 read access to the file named by this abstract pathname

        See
        - FileStore.getUnallocatedSpace

        Since
        - 1.6
        """
        ...


    def getUsableSpace(self) -> int:
        """
        Returns the number of bytes available to this virtual machine on the
        partition <a href="#partName">named</a> by this abstract pathname.  If
        the number of available bytes in the partition is greater than
        Long.MAX_VALUE, then `Long.MAX_VALUE` will be returned.
        When possible, this method checks for write permissions and other
        operating system restrictions and will therefore usually provide a more
        accurate estimate of how much new data can actually be written than
        .getFreeSpace.
        
         The returned number of available bytes is a hint, but not a
        guarantee, that it is possible to use most or any of these bytes.  The
        number of available bytes is most likely to be accurate immediately
        after this call.  It is likely to be made inaccurate by any external
        I/O operations including those made on the system outside of this
        virtual machine.  This method makes no guarantee that write operations
        to this file system will succeed.

        Returns
        - The number of available bytes on the partition or `0L`
                 if the abstract pathname does not name a partition or if this
                 number cannot be obtained.  On systems where this information
                 is not available, this method will be equivalent to a call to
                 .getFreeSpace.

        Raises
        - SecurityException: If a security manager has been installed and it denies
                 RuntimePermission`("getFileSystemAttributes")`
                 or its SecurityManager.checkRead(String) method denies
                 read access to the file named by this abstract pathname

        See
        - FileStore.getUsableSpace

        Since
        - 1.6
        """
        ...


    @staticmethod
    def createTempFile(prefix: str, suffix: str, directory: "File") -> "File":
        """
         Creates a new empty file in the specified directory, using the
        given prefix and suffix strings to generate its name.  If this method
        returns successfully then it is guaranteed that:
        
        <ol>
        -  The file denoted by the returned abstract pathname did not exist
             before this method was invoked, and
        -  Neither this method nor any of its variants will return the same
             abstract pathname again in the current invocation of the virtual
             machine.
        </ol>
        
        This method provides only part of a temporary-file facility.  To arrange
        for a file created by this method to be deleted automatically, use the
        .deleteOnExit method.
        
         The `prefix` argument must be at least three characters
        long.  It is recommended that the prefix be a short, meaningful string
        such as `"hjb"` or `"mail"`.  The
        `suffix` argument may be `null`, in which case the
        suffix `".tmp"` will be used.
        
         To create the new file, the prefix and the suffix may first be
        adjusted to fit the limitations of the underlying platform.  If the
        prefix is too long then it will be truncated, but its first three
        characters will always be preserved.  If the suffix is too long then it
        too will be truncated, but if it begins with a period character
        (`'.'`) then the period and the first three characters
        following it will always be preserved.  Once these adjustments have been
        made the name of the new file will be generated by concatenating the
        prefix, five or more internally-generated characters, and the suffix.
        
         If the `directory` argument is `null` then the
        system-dependent default temporary-file directory will be used.  The
        default temporary-file directory is specified by the system property
        `java.io.tmpdir`.  On UNIX systems the default value of this
        property is typically `"/tmp"` or `"/var/tmp"`; on
        Microsoft Windows systems it is typically `"C:\\WINNT\\TEMP"`.  A different
        value may be given to this system property when the Java virtual machine
        is invoked, but programmatic changes to this property are not guaranteed
        to have any effect upon the temporary directory used by this method.

        Arguments
        - prefix: The prefix string to be used in generating the file's
                           name; must be at least three characters long
        - suffix: The suffix string to be used in generating the file's
                           name; may be `null`, in which case the
                           suffix `".tmp"` will be used
        - directory: The directory in which the file is to be created, or
                           `null` if the default temporary-file
                           directory is to be used

        Returns
        - An abstract pathname denoting a newly-created empty file

        Raises
        - IllegalArgumentException: If the `prefix` argument contains fewer than three
                 characters
        - IOException: If a file could not be created
        - SecurityException: If a security manager exists and its java.lang.SecurityManager.checkWrite(java.lang.String)
                 method does not allow a file to be created

        Since
        - 1.2
        """
        ...


    @staticmethod
    def createTempFile(prefix: str, suffix: str) -> "File":
        """
        Creates an empty file in the default temporary-file directory, using
        the given prefix and suffix to generate its name. Invoking this method
        is equivalent to invoking .createTempFile(java.lang.String,
        java.lang.String, java.io.File)
        createTempFile(prefix,&nbsp;suffix,&nbsp;null).
        
         The java.nio.file.Files.createTempFile(String,String,java.nio.file.attribute.FileAttribute[])
        Files.createTempFile method provides an alternative method to create an
        empty file in the temporary-file directory. Files created by that method
        may have more restrictive access permissions to files created by this
        method and so may be more suited to security-sensitive applications.

        Arguments
        - prefix: The prefix string to be used in generating the file's
                           name; must be at least three characters long
        - suffix: The suffix string to be used in generating the file's
                           name; may be `null`, in which case the
                           suffix `".tmp"` will be used

        Returns
        - An abstract pathname denoting a newly-created empty file

        Raises
        - IllegalArgumentException: If the `prefix` argument contains fewer than three
                 characters
        - IOException: If a file could not be created
        - SecurityException: If a security manager exists and its java.lang.SecurityManager.checkWrite(java.lang.String)
                 method does not allow a file to be created

        See
        - java.nio.file.Files.createTempDirectory(String,FileAttribute[])

        Since
        - 1.2
        """
        ...


    def compareTo(self, pathname: "File") -> int:
        """
        Compares two abstract pathnames lexicographically.  The ordering
        defined by this method depends upon the underlying system.  On UNIX
        systems, alphabetic case is significant in comparing pathnames; on
        Microsoft Windows systems it is not.

        Arguments
        - pathname: The abstract pathname to be compared to this abstract
                           pathname

        Returns
        - Zero if the argument is equal to this abstract pathname, a
                 value less than zero if this abstract pathname is
                 lexicographically less than the argument, or a value greater
                 than zero if this abstract pathname is lexicographically
                 greater than the argument

        Since
        - 1.2
        """
        ...


    def equals(self, obj: "Object") -> bool:
        """
        Tests this abstract pathname for equality with the given object.
        Returns `True` if and only if the argument is not
        `null` and is an abstract pathname that is the same as this
        abstract pathname.  Whether or not two abstract
        pathnames are equal depends upon the underlying operating system.
        On UNIX systems, alphabetic case is significant in comparing pathnames;
        on Microsoft Windows systems it is not.

        Arguments
        - obj: The object to be compared with this abstract pathname

        Returns
        - `True` if and only if the objects are the same;
                 `False` otherwise

        See
        - java.nio.file.Files.isSameFile(Path,Path)

        Unknown Tags
        - This method only tests whether the abstract pathnames are equal;
                 it does not access the file system and the file is not required
                 to exist.
        """
        ...


    def hashCode(self) -> int:
        """
        Computes a hash code for this abstract pathname.  Because equality of
        abstract pathnames is inherently system-dependent, so is the computation
        of their hash codes.  On UNIX systems, the hash code of an abstract
        pathname is equal to the exclusive *or* of the hash code
        of its pathname string and the decimal value
        `1234321`.  On Microsoft Windows systems, the hash
        code is equal to the exclusive *or* of the hash code of
        its pathname string converted to lower case and the decimal
        value `1234321`.  Locale is not taken into account on
        lowercasing the pathname string.

        Returns
        - A hash code for this abstract pathname
        """
        ...


    def toString(self) -> str:
        """
        Returns the pathname string of this abstract pathname.  This is just the
        string returned by the .getPath method.

        Returns
        - The string form of this abstract pathname
        """
        ...


    def toPath(self) -> "Path":
        """
        Returns a Path java.nio.file.Path object constructed from
        this abstract path. The resulting `Path` is associated with the
        java.nio.file.FileSystems.getDefault default-filesystem.
        
         The first invocation of this method works as if invoking it were
        equivalent to evaluating the expression:
        <blockquote>```
        java.nio.file.FileSystems.getDefault FileSystems.getDefault().java.nio.file.FileSystem.getPath getPath(this..getPath getPath());
        ```</blockquote>
        Subsequent invocations of this method return the same `Path`.
        
         If this abstract pathname is the empty abstract pathname then this
        method returns a `Path` that may be used to access the current
        user directory.

        Returns
        - a `Path` constructed from this abstract path

        Raises
        - java.nio.file.InvalidPathException: if a `Path` object cannot be constructed from the abstract
                 path (see java.nio.file.FileSystem.getPath FileSystem.getPath)

        See
        - Path.toFile

        Since
        - 1.7
        """
        ...



"""
Python module generated from Java source file org.bukkit.block.BrushableBlock

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block import *
from org.bukkit.inventory import ItemStack
from org.bukkit.loot import Lootable
from typing import Any, Callable, Iterable, Tuple


class BrushableBlock(Lootable, TileState):
    """
    Represents a captured state of suspicious sand or gravel.
    """

    def getItem(self) -> "ItemStack":
        """
        Get the item which will be revealed when the sand is fully brushed away
        and uncovered.

        Returns
        - the item
        """
        ...


    def setItem(self, item: "ItemStack") -> None:
        """
        Sets the item which will be revealed when the sand is fully brushed away
        and uncovered.

        Arguments
        - item: the item
        """
        ...



"""
Python module generated from Java source file org.bukkit.Instrument

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.collect import Maps
from enum import Enum
from org.bukkit import *
from typing import Any, Callable, Iterable, Tuple


class Instrument(Enum):

    PIANO = (0x0, Sound.BLOCK_NOTE_BLOCK_HARP)
    """
    Piano is the standard instrument for a note block.
    """
    BASS_DRUM = (0x1, Sound.BLOCK_NOTE_BLOCK_BASEDRUM)
    """
    Bass drum is normally played when a note block is on top of a
    stone-like block.
    """
    SNARE_DRUM = (0x2, Sound.BLOCK_NOTE_BLOCK_SNARE)
    """
    Snare drum is normally played when a note block is on top of a sandy
    block.
    """
    STICKS = (0x3, Sound.BLOCK_NOTE_BLOCK_HAT)
    """
    Sticks are normally played when a note block is on top of a glass
    block.
    """
    BASS_GUITAR = (0x4, Sound.BLOCK_NOTE_BLOCK_BASS)
    """
    Bass guitar is normally played when a note block is on top of a wooden
    block.
    """
    FLUTE = (0x5, Sound.BLOCK_NOTE_BLOCK_FLUTE)
    """
    Flute is normally played when a note block is on top of a clay block.
    """
    BELL = (0x6, Sound.BLOCK_NOTE_BLOCK_BELL)
    """
    Bell is normally played when a note block is on top of a gold block.
    """
    GUITAR = (0x7, Sound.BLOCK_NOTE_BLOCK_GUITAR)
    """
    Guitar is normally played when a note block is on top of a woolen block.
    """
    CHIME = (0x8, Sound.BLOCK_NOTE_BLOCK_CHIME)
    """
    Chime is normally played when a note block is on top of a packed ice
    block.
    """
    XYLOPHONE = (0x9, Sound.BLOCK_NOTE_BLOCK_XYLOPHONE)
    """
    Xylophone is normally played when a note block is on top of a bone block.
    """
    IRON_XYLOPHONE = (0xA, Sound.BLOCK_NOTE_BLOCK_IRON_XYLOPHONE)
    """
    Iron Xylophone is normally played when a note block is on top of a iron block.
    """
    COW_BELL = (0xB, Sound.BLOCK_NOTE_BLOCK_COW_BELL)
    """
    Cow Bell is normally played when a note block is on top of a soul sand block.
    """
    DIDGERIDOO = (0xC, Sound.BLOCK_NOTE_BLOCK_DIDGERIDOO)
    """
    Didgeridoo is normally played when a note block is on top of a pumpkin block.
    """
    BIT = (0xD, Sound.BLOCK_NOTE_BLOCK_BIT)
    """
    Bit is normally played when a note block is on top of a emerald block.
    """
    BANJO = (0xE, Sound.BLOCK_NOTE_BLOCK_BANJO)
    """
    Banjo is normally played when a note block is on top of a hay block.
    """
    PLING = (0xF, Sound.BLOCK_NOTE_BLOCK_PLING)
    """
    Pling is normally played when a note block is on top of a glowstone block.
    """
    ZOMBIE = (Sound.BLOCK_NOTE_BLOCK_IMITATE_ZOMBIE)
    """
    Zombie is normally played when a Zombie Head is on top of the note block.
    """
    SKELETON = (Sound.BLOCK_NOTE_BLOCK_IMITATE_SKELETON)
    """
    Skeleton is normally played when a Skeleton Head is on top of the note block.
    """
    CREEPER = (Sound.BLOCK_NOTE_BLOCK_IMITATE_CREEPER)
    """
    Creeper is normally played when a Creeper Head is on top of the note block.
    """
    DRAGON = (Sound.BLOCK_NOTE_BLOCK_IMITATE_ENDER_DRAGON)
    """
    Dragon is normally played when a Dragon Head is on top of the note block.
    """
    WITHER_SKELETON = (Sound.BLOCK_NOTE_BLOCK_IMITATE_WITHER_SKELETON)
    """
    Wither Skeleton is normally played when a Wither Skeleton Head is on top of the note block.
    """
    PIGLIN = (Sound.BLOCK_NOTE_BLOCK_IMITATE_PIGLIN)
    """
    Piglin is normally played when a Piglin Head is on top of the note block.
    """
    CUSTOM_HEAD = (None)
    """
    Custom Sound is normally played when a Player Head with the required data is on top of the note block.
    """


    def getSound(self) -> "Sound":
        """
        Gets the sound associated with this instrument. 
        Will be null for Instrument.CUSTOM_HEAD

        Returns
        - the sound or null
        """
        ...


    def getType(self) -> int:
        """
        Returns
        - The type ID of this instrument.

        Deprecated
        - Magic value
        """
        ...


    @staticmethod
    def getByType(type: int) -> "Instrument":
        """
        Get an instrument by its type ID.

        Arguments
        - type: The type ID

        Returns
        - The instrument

        Deprecated
        - Magic value
        """
        ...



"""
Python module generated from Java source file com.google.common.eventbus.AllowConcurrentEvents

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.eventbus import *
from typing import Any, Callable, Iterable, Tuple


class AllowConcurrentEvents:
    """
    Marks an event subscriber method as being thread-safe. This annotation indicates that EventBus
    may invoke the event subscriber simultaneously from multiple threads.
    
    This does not mark the method, and so should be used in combination with Subscribe.

    Author(s)
    - Cliff Biffle

    Since
    - 10.0
    """





"""
Python module generated from Java source file com.google.common.collect.Queues

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.base import Preconditions
from com.google.common.collect import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from java.time import Duration
from java.util import ArrayDeque
from java.util import Deque
from java.util import PriorityQueue
from java.util import Queue
from java.util.concurrent import ArrayBlockingQueue
from java.util.concurrent import BlockingQueue
from java.util.concurrent import ConcurrentLinkedQueue
from java.util.concurrent import LinkedBlockingDeque
from java.util.concurrent import LinkedBlockingQueue
from java.util.concurrent import PriorityBlockingQueue
from java.util.concurrent import SynchronousQueue
from java.util.concurrent import TimeUnit
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class Queues:
    """
    Static utility methods pertaining to Queue and Deque instances. Also see this
    class's counterparts Lists, Sets, and Maps.

    Author(s)
    - Kurt Alfred Kluever

    Since
    - 11.0
    """

    @staticmethod
    def newArrayBlockingQueue(capacity: int) -> "ArrayBlockingQueue"["E"]:
        """
        Creates an empty `ArrayBlockingQueue` with the given (fixed) capacity and nonfair access
        policy.
        """
        ...


    @staticmethod
    def newArrayDeque() -> "ArrayDeque"["E"]:
        """
        Creates an empty `ArrayDeque`.

        Since
        - 12.0
        """
        ...


    @staticmethod
    def newArrayDeque(elements: Iterable["E"]) -> "ArrayDeque"["E"]:
        """
        Creates an `ArrayDeque` containing the elements of the specified iterable, in the order
        they are returned by the iterable's iterator.

        Since
        - 12.0
        """
        ...


    @staticmethod
    def newConcurrentLinkedQueue() -> "ConcurrentLinkedQueue"["E"]:
        """
        Creates an empty `ConcurrentLinkedQueue`.
        """
        ...


    @staticmethod
    def newConcurrentLinkedQueue(elements: Iterable["E"]) -> "ConcurrentLinkedQueue"["E"]:
        """
        Creates a `ConcurrentLinkedQueue` containing the elements of the specified iterable, in
        the order they are returned by the iterable's iterator.
        """
        ...


    @staticmethod
    def newLinkedBlockingDeque() -> "LinkedBlockingDeque"["E"]:
        """
        Creates an empty `LinkedBlockingDeque` with a capacity of Integer.MAX_VALUE.

        Since
        - 12.0
        """
        ...


    @staticmethod
    def newLinkedBlockingDeque(capacity: int) -> "LinkedBlockingDeque"["E"]:
        """
        Creates an empty `LinkedBlockingDeque` with the given (fixed) capacity.

        Raises
        - IllegalArgumentException: if `capacity` is less than 1

        Since
        - 12.0
        """
        ...


    @staticmethod
    def newLinkedBlockingDeque(elements: Iterable["E"]) -> "LinkedBlockingDeque"["E"]:
        """
        Creates a `LinkedBlockingDeque` with a capacity of Integer.MAX_VALUE, containing
        the elements of the specified iterable, in the order they are returned by the iterable's
        iterator.

        Since
        - 12.0
        """
        ...


    @staticmethod
    def newLinkedBlockingQueue() -> "LinkedBlockingQueue"["E"]:
        """
        Creates an empty `LinkedBlockingQueue` with a capacity of Integer.MAX_VALUE.
        """
        ...


    @staticmethod
    def newLinkedBlockingQueue(capacity: int) -> "LinkedBlockingQueue"["E"]:
        """
        Creates an empty `LinkedBlockingQueue` with the given (fixed) capacity.

        Raises
        - IllegalArgumentException: if `capacity` is less than 1
        """
        ...


    @staticmethod
    def newLinkedBlockingQueue(elements: Iterable["E"]) -> "LinkedBlockingQueue"["E"]:
        """
        Creates a `LinkedBlockingQueue` with a capacity of Integer.MAX_VALUE, containing
        the elements of the specified iterable, in the order they are returned by the iterable's
        iterator.

        Arguments
        - elements: the elements that the queue should contain, in order

        Returns
        - a new `LinkedBlockingQueue` containing those elements
        """
        ...


    @staticmethod
    def newPriorityBlockingQueue() -> "PriorityBlockingQueue"["E"]:
        """
        Creates an empty `PriorityBlockingQueue` with the ordering given by its elements' natural
        ordering.

        Since
        - 11.0 (but the bound of `E` was changed from `Object` to `Comparable`
            in 15.0)
        """
        ...


    @staticmethod
    def newPriorityBlockingQueue(elements: Iterable["E"]) -> "PriorityBlockingQueue"["E"]:
        """
        Creates a `PriorityBlockingQueue` containing the given elements.
        
        **Note:** If the specified iterable is a `SortedSet` or a `PriorityQueue`,
        this priority queue will be ordered according to the same ordering.

        Since
        - 11.0 (but the bound of `E` was changed from `Object` to `Comparable`
            in 15.0)
        """
        ...


    @staticmethod
    def newPriorityQueue() -> "PriorityQueue"["E"]:
        """
        Creates an empty `PriorityQueue` with the ordering given by its elements' natural
        ordering.

        Since
        - 11.0 (but the bound of `E` was changed from `Object` to `Comparable`
            in 15.0)
        """
        ...


    @staticmethod
    def newPriorityQueue(elements: Iterable["E"]) -> "PriorityQueue"["E"]:
        """
        Creates a `PriorityQueue` containing the given elements.
        
        **Note:** If the specified iterable is a `SortedSet` or a `PriorityQueue`,
        this priority queue will be ordered according to the same ordering.

        Since
        - 11.0 (but the bound of `E` was changed from `Object` to `Comparable`
            in 15.0)
        """
        ...


    @staticmethod
    def newSynchronousQueue() -> "SynchronousQueue"["E"]:
        """
        Creates an empty `SynchronousQueue` with nonfair access policy.
        """
        ...


    @staticmethod
    def drain(q: "BlockingQueue"["E"], buffer: Iterable["E"], numElements: int, timeout: "Duration") -> int:
        """
        Drains the queue as BlockingQueue.drainTo(Collection, int), but if the requested `numElements` elements are not available, it will wait for them up to the specified timeout.

        Arguments
        - q: the blocking queue to be drained
        - buffer: where to add the transferred elements
        - numElements: the number of elements to be waited for
        - timeout: how long to wait before giving up

        Returns
        - the number of elements transferred

        Raises
        - InterruptedException: if interrupted while waiting

        Since
        - 28.0
        """
        ...


    @staticmethod
    def drain(q: "BlockingQueue"["E"], buffer: Iterable["E"], numElements: int, timeout: int, unit: "TimeUnit") -> int:
        """
        Drains the queue as BlockingQueue.drainTo(Collection, int), but if the requested `numElements` elements are not available, it will wait for them up to the specified timeout.

        Arguments
        - q: the blocking queue to be drained
        - buffer: where to add the transferred elements
        - numElements: the number of elements to be waited for
        - timeout: how long to wait before giving up, in units of `unit`
        - unit: a `TimeUnit` determining how to interpret the timeout parameter

        Returns
        - the number of elements transferred

        Raises
        - InterruptedException: if interrupted while waiting
        """
        ...


    @staticmethod
    def drainUninterruptibly(q: "BlockingQueue"["E"], buffer: Iterable["E"], numElements: int, timeout: "Duration") -> int:
        """
        Drains the queue as .drain(BlockingQueue, Collection, int, Duration), but with a
        different behavior in case it is interrupted while waiting. In that case, the operation will
        continue as usual, and in the end the thread's interruption status will be set (no `InterruptedException` is thrown).

        Arguments
        - q: the blocking queue to be drained
        - buffer: where to add the transferred elements
        - numElements: the number of elements to be waited for
        - timeout: how long to wait before giving up

        Returns
        - the number of elements transferred

        Since
        - 28.0
        """
        ...


    @staticmethod
    def drainUninterruptibly(q: "BlockingQueue"["E"], buffer: Iterable["E"], numElements: int, timeout: int, unit: "TimeUnit") -> int:
        """
        Drains the queue as .drain(BlockingQueue, Collection, int, long, TimeUnit), but
        with a different behavior in case it is interrupted while waiting. In that case, the operation
        will continue as usual, and in the end the thread's interruption status will be set (no `InterruptedException` is thrown).

        Arguments
        - q: the blocking queue to be drained
        - buffer: where to add the transferred elements
        - numElements: the number of elements to be waited for
        - timeout: how long to wait before giving up, in units of `unit`
        - unit: a `TimeUnit` determining how to interpret the timeout parameter

        Returns
        - the number of elements transferred
        """
        ...


    @staticmethod
    def synchronizedQueue(queue: "Queue"["E"]) -> "Queue"["E"]:
        """
        Returns a synchronized (thread-safe) queue backed by the specified queue. In order to guarantee
        serial access, it is critical that **all** access to the backing queue is accomplished
        through the returned queue.
        
        It is imperative that the user manually synchronize on the returned queue when accessing the
        queue's iterator:
        
        ````Queue<E> queue = Queues.synchronizedQueue(MinMaxPriorityQueue.<E>create());
        ...
        queue.add(element);  // Needn't be in synchronized block
        ...
        synchronized (queue) {  // Must synchronize on queue!
          Iterator<E> i = queue.iterator(); // Must be in synchronized block
          while (i.hasNext()) {
            foo(i.next());`
        }
        }```
        
        Failure to follow this advice may result in non-deterministic behavior.
        
        The returned queue will be serializable if the specified queue is serializable.

        Arguments
        - queue: the queue to be wrapped in a synchronized view

        Returns
        - a synchronized view of the specified queue

        Since
        - 14.0
        """
        ...


    @staticmethod
    def synchronizedDeque(deque: "Deque"["E"]) -> "Deque"["E"]:
        """
        Returns a synchronized (thread-safe) deque backed by the specified deque. In order to guarantee
        serial access, it is critical that **all** access to the backing deque is accomplished
        through the returned deque.
        
        It is imperative that the user manually synchronize on the returned deque when accessing any
        of the deque's iterators:
        
        ````Deque<E> deque = Queues.synchronizedDeque(Queues.<E>newArrayDeque());
        ...
        deque.add(element);  // Needn't be in synchronized block
        ...
        synchronized (deque) {  // Must synchronize on deque!
          Iterator<E> i = deque.iterator(); // Must be in synchronized block
          while (i.hasNext()) {
            foo(i.next());`
        }
        }```
        
        Failure to follow this advice may result in non-deterministic behavior.
        
        The returned deque will be serializable if the specified deque is serializable.

        Arguments
        - deque: the deque to be wrapped in a synchronized view

        Returns
        - a synchronized view of the specified deque

        Since
        - 15.0
        """
        ...



"""
Python module generated from Java source file org.bukkit.material.Rails

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Material
from org.bukkit.block import BlockFace
from org.bukkit.material import *
from typing import Any, Callable, Iterable, Tuple


class Rails(MaterialData):
    """
    Represents minecart rails.

    Deprecated
    - all usage of MaterialData is deprecated and subject to removal.
    Use org.bukkit.block.data.BlockData.
    """

    def __init__(self):
        ...


    def __init__(self, type: "Material"):
        ...


    def __init__(self, type: "Material", data: int):
        """
        Arguments
        - type: the type
        - data: the raw data value

        Deprecated
        - Magic value
        """
        ...


    def isOnSlope(self) -> bool:
        """
        Returns
        - the whether this track is set on a slope
        """
        ...


    def isCurve(self) -> bool:
        """
        Returns
        - the whether this track is set as a curve
        """
        ...


    def getDirection(self) -> "BlockFace":
        """
        Returns
        - the direction these tracks are set
            
            Note that tracks are bidirectional and that the direction returned
            is the ascending direction if the track is set on a slope. If it is
            set as a curve, the corner of the track is returned.
        """
        ...


    def toString(self) -> str:
        ...


    def setDirection(self, face: "BlockFace", isOnSlope: bool) -> None:
        """
        Set the direction of these tracks
        
        Note that tracks are bidirectional and that the direction returned is
        the ascending direction if the track is set on a slope. If it is set as
        a curve, the corner of the track should be supplied.

        Arguments
        - face: the direction the track should be facing
        - isOnSlope: whether or not the track should be on a slope
        """
        ...


    def clone(self) -> "Rails":
        ...



from .GsonBuildConfig import GsonBuildConfig
from .UnsafeAllocator import UnsafeAllocator
from .TroubleshootingGuide import TroubleshootingGuide
from .Streams import Streams
from .ReflectionAccessFilterHelper import ReflectionAccessFilterHelper
from .Primitives import Primitives
from .PreJava9DateFormatProvider import PreJava9DateFormatProvider
from .ObjectConstructor import ObjectConstructor
from .NumberLimits import NumberLimits
from .NonNullElementWrapperList import NonNullElementWrapperList
from .LinkedTreeMap import LinkedTreeMap
from .LazilyParsedNumber import LazilyParsedNumber
from .JsonReaderInternalAccess import JsonReaderInternalAccess
from .JavaVersion import JavaVersion
from .Excluder import Excluder
from .ConstructorConstructor import ConstructorConstructor
from .$Gson$Types import $Gson$Types
from .$Gson$Preconditions import $Gson$Preconditions


"""
Python module generated from Java source file java.util.concurrent.locks.LockSupport

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.util.concurrent.locks import *
from jdk.internal.misc import Unsafe
from typing import Any, Callable, Iterable, Tuple


class LockSupport:
    """
    Basic thread blocking primitives for creating locks and other
    synchronization classes.
    
    This class associates, with each thread that uses it, a permit
    (in the sense of the java.util.concurrent.Semaphore
    Semaphore class). A call to `park` will return immediately
    if the permit is available, consuming it in the process; otherwise
    it *may* block.  A call to `unpark` makes the permit
    available, if it was not already available. (Unlike with Semaphores
    though, permits do not accumulate. There is at most one.)
    Reliable usage requires the use of volatile (or atomic) variables
    to control when to park or unpark.  Orderings of calls to these
    methods are maintained with respect to volatile variable accesses,
    but not necessarily non-volatile variable accesses.
    
    Methods `park` and `unpark` provide efficient
    means of blocking and unblocking threads that do not encounter the
    problems that cause the deprecated methods `Thread.suspend`
    and `Thread.resume` to be unusable for such purposes: Races
    between one thread invoking `park` and another thread trying
    to `unpark` it will preserve liveness, due to the
    permit. Additionally, `park` will return if the caller's
    thread was interrupted, and timeout versions are supported. The
    `park` method may also return at any other time, for "no
    reason", so in general must be invoked within a loop that rechecks
    conditions upon return. In this sense `park` serves as an
    optimization of a "busy wait" that does not waste as much time
    spinning, but must be paired with an `unpark` to be
    effective.
    
    The three forms of `park` each also support a
    `blocker` object parameter. This object is recorded while
    the thread is blocked to permit monitoring and diagnostic tools to
    identify the reasons that threads are blocked. (Such tools may
    access blockers using method .getBlocker(Thread).)
    The use of these forms rather than the original forms without this
    parameter is strongly encouraged. The normal argument to supply as
    a `blocker` within a lock implementation is `this`.
    
    These methods are designed to be used as tools for creating
    higher-level synchronization utilities, and are not in themselves
    useful for most concurrency control applications.  The `park`
    method is designed for use only in constructions of the form:
    
    ``` `while (!canProceed()) {
      // ensure request to unpark is visible to other threads
      ...
      LockSupport.park(this);`}```
    
    where no actions by the thread publishing a request to unpark,
    prior to the call to `park`, entail locking or blocking.
    Because only one permit is associated with each thread, any
    intermediary uses of `park`, including implicitly via class
    loading, could lead to an unresponsive thread (a "lost unpark").
    
    **Sample Usage.** Here is a sketch of a first-in-first-out
    non-reentrant lock class:
    ``` `class FIFOMutex {
      private final AtomicBoolean locked = new AtomicBoolean(False);
      private final Queue<Thread> waiters
        = new ConcurrentLinkedQueue<>();
    
      public void lock() {
        boolean wasInterrupted = False;
        // publish current thread for unparkers
        waiters.add(Thread.currentThread());
    
        // Block while not first in queue or cannot acquire lock
        while (waiters.peek() != Thread.currentThread() ||
               !locked.compareAndSet(False, True)) {
          LockSupport.park(this);
          // ignore interrupts while waiting
          if (Thread.interrupted())
            wasInterrupted = True;`
    
        waiters.remove();
        // ensure correct interrupt status on return
        if (wasInterrupted)
          Thread.currentThread().interrupt();
      }
    
      public void unlock() {
        locked.set(False);
        LockSupport.unpark(waiters.peek());
      }
    
      static {
        // Reduce the risk of "lost unpark" due to classloading
        Class<?> ensureLoaded = LockSupport.class;
      }
    }}```

    Since
    - 1.5
    """

    @staticmethod
    def setCurrentBlocker(blocker: "Object") -> None:
        """
        Sets the object to be returned by invocations of .getBlocker getBlocker for the current thread. This method may
        be used before invoking the no-argument version of LockSupport.park() park() from non-public objects, allowing
        more helpful diagnostics, or retaining compatibility with
        previous implementations of blocking methods.  Previous values
        of the blocker are not automatically restored after blocking.
        To obtain the effects of `park(b`}, use `setCurrentBlocker(b); park(); setCurrentBlocker(null);`

        Arguments
        - blocker: the blocker object

        Since
        - 14
        """
        ...


    @staticmethod
    def unpark(thread: "Thread") -> None:
        """
        Makes available the permit for the given thread, if it
        was not already available.  If the thread was blocked on
        `park` then it will unblock.  Otherwise, its next call
        to `park` is guaranteed not to block. This operation
        is not guaranteed to have any effect at all if the given
        thread has not been started.

        Arguments
        - thread: the thread to unpark, or `null`, in which case
               this operation has no effect
        """
        ...


    @staticmethod
    def park(blocker: "Object") -> None:
        """
        Disables the current thread for thread scheduling purposes unless the
        permit is available.
        
        If the permit is available then it is consumed and the call returns
        immediately; otherwise
        the current thread becomes disabled for thread scheduling
        purposes and lies dormant until one of three things happens:
        
        
        - Some other thread invokes .unpark unpark with the
        current thread as the target; or
        
        - Some other thread Thread.interrupt interrupts
        the current thread; or
        
        - The call spuriously (that is, for no reason) returns.
        
        
        This method does *not* report which of these caused the
        method to return. Callers should re-check the conditions which caused
        the thread to park in the first place. Callers may also determine,
        for example, the interrupt status of the thread upon return.

        Arguments
        - blocker: the synchronization object responsible for this
               thread parking

        Since
        - 1.6
        """
        ...


    @staticmethod
    def parkNanos(blocker: "Object", nanos: int) -> None:
        """
        Disables the current thread for thread scheduling purposes, for up to
        the specified waiting time, unless the permit is available.
        
        If the specified waiting time is zero or negative, the
        method does nothing. Otherwise, if the permit is available then
        it is consumed and the call returns immediately; otherwise the
        current thread becomes disabled for thread scheduling purposes
        and lies dormant until one of four things happens:
        
        
        - Some other thread invokes .unpark unpark with the
        current thread as the target; or
        
        - Some other thread Thread.interrupt interrupts
        the current thread; or
        
        - The specified waiting time elapses; or
        
        - The call spuriously (that is, for no reason) returns.
        
        
        This method does *not* report which of these caused the
        method to return. Callers should re-check the conditions which caused
        the thread to park in the first place. Callers may also determine,
        for example, the interrupt status of the thread, or the elapsed time
        upon return.

        Arguments
        - blocker: the synchronization object responsible for this
               thread parking
        - nanos: the maximum number of nanoseconds to wait

        Since
        - 1.6
        """
        ...


    @staticmethod
    def parkUntil(blocker: "Object", deadline: int) -> None:
        """
        Disables the current thread for thread scheduling purposes, until
        the specified deadline, unless the permit is available.
        
        If the permit is available then it is consumed and the call
        returns immediately; otherwise the current thread becomes disabled
        for thread scheduling purposes and lies dormant until one of four
        things happens:
        
        
        - Some other thread invokes .unpark unpark with the
        current thread as the target; or
        
        - Some other thread Thread.interrupt interrupts the
        current thread; or
        
        - The specified deadline passes; or
        
        - The call spuriously (that is, for no reason) returns.
        
        
        This method does *not* report which of these caused the
        method to return. Callers should re-check the conditions which caused
        the thread to park in the first place. Callers may also determine,
        for example, the interrupt status of the thread, or the current time
        upon return.

        Arguments
        - blocker: the synchronization object responsible for this
               thread parking
        - deadline: the absolute time, in milliseconds from the Epoch,
               to wait until

        Since
        - 1.6
        """
        ...


    @staticmethod
    def getBlocker(t: "Thread") -> "Object":
        """
        Returns the blocker object supplied to the most recent
        invocation of a park method that has not yet unblocked, or null
        if not blocked.  The value returned is just a momentary
        snapshot -- the thread may have since unblocked or blocked on a
        different blocker object.

        Arguments
        - t: the thread

        Returns
        - the blocker

        Raises
        - NullPointerException: if argument is null

        Since
        - 1.6
        """
        ...


    @staticmethod
    def park() -> None:
        """
        Disables the current thread for thread scheduling purposes unless the
        permit is available.
        
        If the permit is available then it is consumed and the call
        returns immediately; otherwise the current thread becomes disabled
        for thread scheduling purposes and lies dormant until one of three
        things happens:
        
        
        
        - Some other thread invokes .unpark unpark with the
        current thread as the target; or
        
        - Some other thread Thread.interrupt interrupts
        the current thread; or
        
        - The call spuriously (that is, for no reason) returns.
        
        
        This method does *not* report which of these caused the
        method to return. Callers should re-check the conditions which caused
        the thread to park in the first place. Callers may also determine,
        for example, the interrupt status of the thread upon return.
        """
        ...


    @staticmethod
    def parkNanos(nanos: int) -> None:
        """
        Disables the current thread for thread scheduling purposes, for up to
        the specified waiting time, unless the permit is available.
        
        If the specified waiting time is zero or negative, the
        method does nothing. Otherwise, if the permit is available then
        it is consumed and the call returns immediately; otherwise the
        current thread becomes disabled for thread scheduling purposes
        and lies dormant until one of four things happens:
        
        
        - Some other thread invokes .unpark unpark with the
        current thread as the target; or
        
        - Some other thread Thread.interrupt interrupts
        the current thread; or
        
        - The specified waiting time elapses; or
        
        - The call spuriously (that is, for no reason) returns.
        
        
        This method does *not* report which of these caused the
        method to return. Callers should re-check the conditions which caused
        the thread to park in the first place. Callers may also determine,
        for example, the interrupt status of the thread, or the elapsed time
        upon return.

        Arguments
        - nanos: the maximum number of nanoseconds to wait
        """
        ...


    @staticmethod
    def parkUntil(deadline: int) -> None:
        """
        Disables the current thread for thread scheduling purposes, until
        the specified deadline, unless the permit is available.
        
        If the permit is available then it is consumed and the call
        returns immediately; otherwise the current thread becomes disabled
        for thread scheduling purposes and lies dormant until one of four
        things happens:
        
        
        - Some other thread invokes .unpark unpark with the
        current thread as the target; or
        
        - Some other thread Thread.interrupt interrupts
        the current thread; or
        
        - The specified deadline passes; or
        
        - The call spuriously (that is, for no reason) returns.
        
        
        This method does *not* report which of these caused the
        method to return. Callers should re-check the conditions which caused
        the thread to park in the first place. Callers may also determine,
        for example, the interrupt status of the thread, or the current time
        upon return.

        Arguments
        - deadline: the absolute time, in milliseconds from the Epoch,
               to wait until
        """
        ...



"""
Python module generated from Java source file org.bukkit.entity.ItemFrame

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit import Rotation
from org.bukkit.entity import *
from org.bukkit.inventory import ItemStack
from typing import Any, Callable, Iterable, Tuple


class ItemFrame(Hanging):
    """
    Represents an Item Frame
    """

    def getItem(self) -> "ItemStack":
        """
        Get the item in this frame

        Returns
        - a defensive copy the item in this item frame
        """
        ...


    def setItem(self, item: "ItemStack") -> None:
        """
        Set the item in this frame

        Arguments
        - item: the new item
        """
        ...


    def setItem(self, item: "ItemStack", playSound: bool) -> None:
        """
        Set the item in this frame

        Arguments
        - item: the new item
        - playSound: whether or not to play the item placement sound
        """
        ...


    def getItemDropChance(self) -> float:
        """
        Gets the chance of the item being dropped upon this frame's destruction.
        
        
        - A drop chance of 0.0F will never drop
        - A drop chance of 1.0F will always drop

        Returns
        - chance of the off hand item being dropped
        """
        ...


    def setItemDropChance(self, chance: float) -> None:
        """
        Sets the chance of the off hand item being dropped upon this frame's
        destruction.
        
        
        - A drop chance of 0.0F will never drop
        - A drop chance of 1.0F will always drop

        Arguments
        - chance: the chance of off hand item being dropped
        """
        ...


    def getRotation(self) -> "Rotation":
        """
        Get the rotation of the frame's item

        Returns
        - the direction
        """
        ...


    def setRotation(self, rotation: "Rotation") -> None:
        """
        Set the rotation of the frame's item

        Arguments
        - rotation: the new rotation

        Raises
        - IllegalArgumentException: if rotation is null
        """
        ...


    def isVisible(self) -> bool:
        """
        Returns whether the item frame is be visible or not.

        Returns
        - whether the item frame is visible or not
        """
        ...


    def setVisible(self, visible: bool) -> None:
        """
        Sets whether the item frame should be visible or not.

        Arguments
        - visible: whether the item frame is visible or not
        """
        ...


    def isFixed(self) -> bool:
        """
        Returns whether the item frame is "fixed" or not.
        
        When True it's not possible to destroy/move the frame (e.g. by damage,
        interaction, pistons, or missing supporting blocks), rotate the item or
        place/remove items.

        Returns
        - whether the item frame is fixed or not
        """
        ...


    def setFixed(self, fixed: bool) -> None:
        """
        Sets whether the item frame should be fixed or not.
        
        When set to True it's not possible to destroy/move the frame (e.g. by
        damage, interaction, pistons, or missing supporting blocks), rotate the
        item or place/remove items.

        Arguments
        - fixed: whether the item frame is fixed or not
        """
        ...



"""
Python module generated from Java source file com.google.common.util.concurrent.FakeTimeLimiter

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.util.concurrent import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from java.util.concurrent import Callable
from java.util.concurrent import ExecutionException
from java.util.concurrent import TimeUnit
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class FakeTimeLimiter(TimeLimiter):
    """
    A TimeLimiter implementation which actually does not attempt to limit time at all. This may be
    desirable to use in some unit tests. More importantly, attempting to debug a call which is
    time-limited would be extremely annoying, so this gives you a time-limiter you can easily swap in
    for your real time-limiter while you're debugging.

    Author(s)
    - Jens Nyman

    Since
    - 1.0
    """

    def newProxy(self, target: "T", interfaceType: type["T"], timeoutDuration: int, timeoutUnit: "TimeUnit") -> "T":
        ...


    def callWithTimeout(self, callable: "Callable"["T"], timeoutDuration: int, timeoutUnit: "TimeUnit") -> "T":
        ...


    def callUninterruptiblyWithTimeout(self, callable: "Callable"["T"], timeoutDuration: int, timeoutUnit: "TimeUnit") -> "T":
        ...


    def runWithTimeout(self, runnable: "Runnable", timeoutDuration: int, timeoutUnit: "TimeUnit") -> None:
        ...


    def runUninterruptiblyWithTimeout(self, runnable: "Runnable", timeoutDuration: int, timeoutUnit: "TimeUnit") -> None:
        ...



"""
Python module generated from Java source file com.google.common.util.concurrent.AtomicDouble

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.util.concurrent import *
from com.google.errorprone.annotations import CanIgnoreReturnValue
from com.google.j2objc.annotations import ReflectionSupport
from java.io import IOException
from java.io import ObjectInputStream
from java.io import ObjectOutputStream
from java.io import Serializable
from java.util.concurrent.atomic import AtomicLongFieldUpdater
from java.util.function import DoubleBinaryOperator
from java.util.function import DoubleUnaryOperator
from typing import Any, Callable, Iterable, Tuple


class AtomicDouble(Number, Serializable):
    """
    A `double` value that may be updated atomically. See the java.util.concurrent.atomic package specification for description of the properties of atomic
    variables. An `AtomicDouble` is used in applications such as atomic accumulation, and
    cannot be used as a replacement for a Double. However, this class does extend `Number` to allow uniform access by tools and utilities that deal with numerically-based classes.
    
    <a id="bitEquals"></a>This class compares primitive `double` values in methods such as
    .compareAndSet by comparing their bitwise representation using Double.doubleToRawLongBits, which differs from both the primitive double `==` operator and
    from Double.equals, as if implemented by:
    
    ````static boolean bitEquals(double x, double y) {
      long xBits = Double.doubleToRawLongBits(x);
      long yBits = Double.doubleToRawLongBits(y);
      return xBits == yBits;`
    }```
    
    It is possible to write a more scalable updater, at the cost of giving up strict atomicity.
    See for example <a
    href="http://gee.cs.oswego.edu/dl/jsr166/dist/docs/java.base/java/util/concurrent/atomic/DoubleAdder.html">
    DoubleAdder</a>.

    Author(s)
    - Martin Buchholz

    Since
    - 11.0
    """

    def __init__(self, initialValue: float):
        """
        Creates a new `AtomicDouble` with the given initial value.

        Arguments
        - initialValue: the initial value
        """
        ...


    def __init__(self):
        """
        Creates a new `AtomicDouble` with initial value `0.0`.
        """
        ...


    def get(self) -> float:
        """
        Gets the current value.

        Returns
        - the current value
        """
        ...


    def set(self, newValue: float) -> None:
        """
        Sets to the given value.

        Arguments
        - newValue: the new value
        """
        ...


    def lazySet(self, newValue: float) -> None:
        """
        Eventually sets to the given value.

        Arguments
        - newValue: the new value
        """
        ...


    def getAndSet(self, newValue: float) -> float:
        """
        Atomically sets to the given value and returns the old value.

        Arguments
        - newValue: the new value

        Returns
        - the previous value
        """
        ...


    def compareAndSet(self, expect: float, update: float) -> bool:
        """
        Atomically sets the value to the given updated value if the current value is <a
        href="#bitEquals">bitwise equal</a> to the expected value.

        Arguments
        - expect: the expected value
        - update: the new value

        Returns
        - `True` if successful. False return indicates that the actual value was not
            bitwise equal to the expected value.
        """
        ...


    def weakCompareAndSet(self, expect: float, update: float) -> bool:
        """
        Atomically sets the value to the given updated value if the current value is <a
        href="#bitEquals">bitwise equal</a> to the expected value.
        
        May <a
        href="http://download.oracle.com/javase/7/docs/api/java/util/concurrent/atomic/package-summary.html#Spurious">
        fail spuriously</a> and does not provide ordering guarantees, so is only rarely an appropriate
        alternative to `compareAndSet`.

        Arguments
        - expect: the expected value
        - update: the new value

        Returns
        - `True` if successful
        """
        ...


    def getAndAdd(self, delta: float) -> float:
        """
        Atomically adds the given value to the current value.

        Arguments
        - delta: the value to add

        Returns
        - the previous value
        """
        ...


    def addAndGet(self, delta: float) -> float:
        """
        Atomically adds the given value to the current value.

        Arguments
        - delta: the value to add

        Returns
        - the updated value
        """
        ...


    def getAndAccumulate(self, x: float, accumulatorFunction: "DoubleBinaryOperator") -> float:
        """
        Atomically updates the current value with the results of applying the given function to the
        current and given values.

        Arguments
        - x: the update value
        - accumulatorFunction: the accumulator function

        Returns
        - the previous value

        Since
        - 31.1
        """
        ...


    def accumulateAndGet(self, x: float, accumulatorFunction: "DoubleBinaryOperator") -> float:
        """
        Atomically updates the current value with the results of applying the given function to the
        current and given values.

        Arguments
        - x: the update value
        - accumulatorFunction: the accumulator function

        Returns
        - the updated value

        Since
        - 31.1
        """
        ...


    def getAndUpdate(self, updateFunction: "DoubleUnaryOperator") -> float:
        """
        Atomically updates the current value with the results of applying the given function.

        Arguments
        - updateFunction: the update function

        Returns
        - the previous value

        Since
        - 31.1
        """
        ...


    def updateAndGet(self, updateFunction: "DoubleUnaryOperator") -> float:
        """
        Atomically updates the current value with the results of applying the given function.

        Arguments
        - updateFunction: the update function

        Returns
        - the updated value

        Since
        - 31.1
        """
        ...


    def toString(self) -> str:
        """
        Returns the String representation of the current value.

        Returns
        - the String representation of the current value
        """
        ...


    def intValue(self) -> int:
        """
        Returns the value of this `AtomicDouble` as an `int` after a narrowing primitive
        conversion.
        """
        ...


    def longValue(self) -> int:
        """
        Returns the value of this `AtomicDouble` as a `long` after a narrowing primitive
        conversion.
        """
        ...


    def floatValue(self) -> float:
        """
        Returns the value of this `AtomicDouble` as a `float` after a narrowing primitive
        conversion.
        """
        ...


    def doubleValue(self) -> float:
        """
        Returns the value of this `AtomicDouble` as a `double`.
        """
        ...



"""
Python module generated from Java source file org.bukkit.inventory.meta.BlockStateMeta

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block import BlockState
from org.bukkit.inventory.meta import *
from typing import Any, Callable, Iterable, Tuple


class BlockStateMeta(ItemMeta):

    def hasBlockState(self) -> bool:
        """
        Returns whether the item has a block state currently
        attached to it.

        Returns
        - whether a block state is already attached
        """
        ...


    def getBlockState(self) -> "BlockState":
        """
        Returns the currently attached block state for this
        item or creates a new one if one doesn't exist.
        
        The state is a copy, it must be set back (or to another
        item) with .setBlockState(org.bukkit.block.BlockState)

        Returns
        - the attached state or a new state
        """
        ...


    def setBlockState(self, blockState: "BlockState") -> None:
        """
        Attaches a copy of the passed block state to the item.

        Arguments
        - blockState: the block state to attach to the block.

        Raises
        - IllegalArgumentException: if the blockState is null
                or invalid for this item.
        """
        ...



"""
Python module generated from Java source file com.google.common.hash.AbstractNonStreamingHashFunction

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from com.google.common.hash import *
from com.google.errorprone.annotations import Immutable
from java.io import ByteArrayOutputStream
from java.nio.charset import Charset
from java.util import Arrays
from typing import Any, Callable, Iterable, Tuple


class AbstractNonStreamingHashFunction(AbstractHashFunction):
    """
    Skeleton implementation of HashFunction, appropriate for non-streaming algorithms. All
    the hash computation done using .newHasher() are delegated to the .hashBytes(byte[], int, int) method.

    Author(s)
    - Dimitris Andreou
    """

    def newHasher(self) -> "Hasher":
        ...


    def newHasher(self, expectedInputSize: int) -> "Hasher":
        ...


    def hashInt(self, input: int) -> "HashCode":
        ...


    def hashLong(self, input: int) -> "HashCode":
        ...


    def hashUnencodedChars(self, input: "CharSequence") -> "HashCode":
        ...


    def hashString(self, input: "CharSequence", charset: "Charset") -> "HashCode":
        ...


    def hashBytes(self, input: list[int], off: int, len: int) -> "HashCode":
        ...


    def hashBytes(self, input: "ByteBuffer") -> "HashCode":
        ...



"""
Python module generated from Java source file org.bukkit.event.player.PlayerUnleashEntityEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.entity import Entity
from org.bukkit.entity import Player
from org.bukkit.event import Cancellable
from org.bukkit.event.entity import EntityUnleashEvent
from org.bukkit.event.player import *
from org.bukkit.inventory import EquipmentSlot
from typing import Any, Callable, Iterable, Tuple


class PlayerUnleashEntityEvent(EntityUnleashEvent, Cancellable):
    """
    Called prior to an entity being unleashed due to a player's action.
    """

    def __init__(self, entity: "Entity", player: "Player", hand: "EquipmentSlot"):
        ...


    def __init__(self, entity: "Entity", player: "Player"):
        ...


    def getPlayer(self) -> "Player":
        """
        Returns the player who is unleashing the entity.

        Returns
        - The player
        """
        ...


    def getHand(self) -> "EquipmentSlot":
        """
        Get the hand used by the player to unleash the entity.

        Returns
        - the hand
        """
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...



"""
Python module generated from Java source file org.bukkit.block.data.type.Sapling

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from org.bukkit.block.data import BlockData
from org.bukkit.block.data.type import *
from typing import Any, Callable, Iterable, Tuple


class Sapling(BlockData):
    """
    'stage' represents the growth stage of a sapling.
    
    When the sapling reaches .getMaximumStage() it will attempt to grow
    into a tree as the next stage.
    """

    def getStage(self) -> int:
        """
        Gets the value of the 'stage' property.

        Returns
        - the 'stage' value
        """
        ...


    def setStage(self, stage: int) -> None:
        """
        Sets the value of the 'stage' property.

        Arguments
        - stage: the new 'stage' value
        """
        ...


    def getMaximumStage(self) -> int:
        """
        Gets the maximum allowed value of the 'stage' property.

        Returns
        - the maximum 'stage' value
        """
        ...



"""
Python module generated from Java source file java.io.DataInput

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import *
from typing import Any, Callable, Iterable, Tuple


class DataInput:
    """
    The `DataInput` interface provides
    for reading bytes from a binary stream and
    reconstructing from them data in any of
    the Java primitive types. There is also
    a
    facility for reconstructing a `String`
    from data in
    <a href="#modified-utf-8">modified UTF-8</a>
    format.
    
    It is generally True of all the reading
    routines in this interface that if end of
    file is reached before the desired number
    of bytes has been read, an `EOFException`
    (which is a kind of `IOException`)
    is thrown. If any byte cannot be read for
    any reason other than end of file, an `IOException`
    other than `EOFException` is
    thrown. In particular, an `IOException`
    may be thrown if the input stream has been
    closed.
    
    <h2><a id="modified-utf-8">Modified UTF-8</a></h2>
    
    Implementations of the DataInput and DataOutput interfaces represent
    Unicode strings in a format that is a slight modification of UTF-8.
    (For information regarding the standard UTF-8 format, see section
    *3.9 Unicode Encoding Forms* of *The Unicode Standard, Version
    4.0*)
    
    
    - Characters in the range `'\u005Cu0001'` to
            `'\u005Cu007F'` are represented by a single byte.
    - The null character `'\u005Cu0000'` and characters
            in the range `'\u005Cu0080'` to `'\u005Cu07FF'` are
            represented by a pair of bytes.
    - Characters in the range `'\u005Cu0800'`
            to `'\u005CuFFFF'` are represented by three bytes.
    
    
      <table class="plain" style="margin-left:2em;">
        <caption>Encoding of UTF-8 values</caption>
        <thead>
        <tr>
          <th scope="col" rowspan="2">Value</th>
          <th scope="col" rowspan="2">Byte</th>
          <th scope="col" colspan="8" id="bit_a">Bit Values</th>
        </tr>
        <tr>
          <!-- Value -->
          <!-- Byte -->
          <th scope="col" style="width:3em"> 7 </th>
          <th scope="col" style="width:3em"> 6 </th>
          <th scope="col" style="width:3em"> 5 </th>
          <th scope="col" style="width:3em"> 4 </th>
          <th scope="col" style="width:3em"> 3 </th>
          <th scope="col" style="width:3em"> 2 </th>
          <th scope="col" style="width:3em"> 1 </th>
          <th scope="col" style="width:3em"> 0 </th>
        </thead>
        <tbody>
        <tr>
          <th scope="row" style="text-align:left; font-weight:normal">
            `\u005Cu0001` to `\u005Cu007F` </th>
          <th scope="row" style="font-weight:normal; text-align:center"> 1 </th>
          <td style="text-align:center">0
          <td colspan="7" style="text-align:right; padding-right:6em">bits 6-0
        </tr>
        <tr>
          <th scope="row" rowspan="2" style="text-align:left; font-weight:normal">
              `\u005Cu0000`,
              `\u005Cu0080` to `\u005Cu07FF` </th>
          <th scope="row" style="font-weight:normal; text-align:center"> 1 </th>
          <td style="text-align:center">1
          <td style="text-align:center">1
          <td style="text-align:center">0
          <td colspan="5" style="text-align:right; padding-right:6em">bits 10-6
        </tr>
        <tr>
          <!-- (value) -->
          <th scope="row" style="font-weight:normal; text-align:center"> 2 </th>
          <td style="text-align:center">1
          <td style="text-align:center">0
          <td colspan="6" style="text-align:right; padding-right:6em">bits 5-0
        </tr>
        <tr>
          <th scope="row" rowspan="3" style="text-align:left; font-weight:normal">
            `\u005Cu0800` to `\u005CuFFFF` </th>
          <th scope="row" style="font-weight:normal; text-align:center"> 1 </th>
          <td style="text-align:center">1
          <td style="text-align:center">1
          <td style="text-align:center">1
          <td style="text-align:center">0
          <td colspan="4" style="text-align:right; padding-right:6em">bits 15-12
        </tr>
        <tr>
          <!-- (value) -->
          <th scope="row" style="font-weight:normal; text-align:center"> 2 </th>
          <td style="text-align:center">1
          <td style="text-align:center">0
          <td colspan="6" style="text-align:right; padding-right:6em">bits 11-6
        </tr>
        <tr>
          <!-- (value) -->
          <th scope="row" style="font-weight:normal; text-align:center"> 3 </th>
          <td style="text-align:center">1
          <td style="text-align:center">0
          <td colspan="6" style="text-align:right; padding-right:6em">bits 5-0
        </tr>
        </tbody>
      </table>
    
    
    The differences between this format and the
    standard UTF-8 format are the following:
    
    - The null byte `'\u005Cu0000'` is encoded in 2-byte format
        rather than 1-byte, so that the encoded strings never have
        embedded nulls.
    - Only the 1-byte, 2-byte, and 3-byte formats are used.
    - <a href="../lang/Character.html#unicode">Supplementary characters</a>
        are represented in the form of surrogate pairs.

    Author(s)
    - Frank Yellin

    See
    - java.io.DataOutput

    Since
    - 1.0
    """

    def readFully(self, b: list[int]) -> None:
        """
        Reads some bytes from an input
        stream and stores them into the buffer
        array `b`. The number of bytes
        read is equal
        to the length of `b`.
        
        This method blocks until one of the
        following conditions occurs:
        
        - `b.length`
        bytes of input data are available, in which
        case a normal return is made.
        
        - End of
        file is detected, in which case an `EOFException`
        is thrown.
        
        - An I/O error occurs, in
        which case an `IOException` other
        than `EOFException` is thrown.
        
        
        If `b` is `null`,
        a `NullPointerException` is thrown.
        If `b.length` is zero, then
        no bytes are read. Otherwise, the first
        byte read is stored into element `b[0]`,
        the next one into `b[1]`, and
        so on.
        If an exception is thrown from
        this method, then it may be that some but
        not all bytes of `b` have been
        updated with data from the input stream.

        Arguments
        - b: the buffer into which the data is read.

        Raises
        - NullPointerException: if `b` is `null`.
        - EOFException: if this stream reaches the end before reading
                 all the bytes.
        - IOException: if an I/O error occurs.
        """
        ...


    def readFully(self, b: list[int], off: int, len: int) -> None:
        """
        Reads `len`
        bytes from
        an input stream.
        
        This method
        blocks until one of the following conditions
        occurs:
        
        - `len` bytes
        of input data are available, in which case
        a normal return is made.
        
        - End of file
        is detected, in which case an `EOFException`
        is thrown.
        
        - An I/O error occurs, in
        which case an `IOException` other
        than `EOFException` is thrown.
        
        
        If `b` is `null`,
        a `NullPointerException` is thrown.
        If `off` is negative, or `len`
        is negative, or `off+len` is
        greater than the length of the array `b`,
        then an `IndexOutOfBoundsException`
        is thrown.
        If `len` is zero,
        then no bytes are read. Otherwise, the first
        byte read is stored into element `b[off]`,
        the next one into `b[off+1]`,
        and so on. The number of bytes read is,
        at most, equal to `len`.

        Arguments
        - b: the buffer into which the data is read.
        - off: an int specifying the offset in the data array `b`.
        - len: an int specifying the number of bytes to read.

        Raises
        - NullPointerException: if `b` is `null`.
        - IndexOutOfBoundsException: if `off` is negative,
                 `len` is negative, or `len` is greater than
                 `b.length - off`.
        - EOFException: if this stream reaches the end before reading
                 all the bytes.
        - IOException: if an I/O error occurs.
        """
        ...


    def skipBytes(self, n: int) -> int:
        """
        Makes an attempt to skip over
        `n` bytes
        of data from the input
        stream, discarding the skipped bytes. However,
        it may skip
        over some smaller number of
        bytes, possibly zero. This may result from
        any of a
        number of conditions; reaching
        end of file before `n` bytes
        have been skipped is
        only one possibility.
        This method never throws an `EOFException`.
        The actual
        number of bytes skipped is returned.

        Arguments
        - n: the number of bytes to be skipped.

        Returns
        - the number of bytes actually skipped.

        Raises
        - IOException: if an I/O error occurs.
        """
        ...


    def readBoolean(self) -> bool:
        """
        Reads one input byte and returns
        `True` if that byte is nonzero,
        `False` if that byte is zero.
        This method is suitable for reading
        the byte written by the `writeBoolean`
        method of interface `DataOutput`.

        Returns
        - the `boolean` value read.

        Raises
        - EOFException: if this stream reaches the end before reading
                      all the bytes.
        - IOException: if an I/O error occurs.
        """
        ...


    def readByte(self) -> int:
        """
        Reads and returns one input byte.
        The byte is treated as a signed value in
        the range `-128` through `127`,
        inclusive.
        This method is suitable for
        reading the byte written by the `writeByte`
        method of interface `DataOutput`.

        Returns
        - the 8-bit value read.

        Raises
        - EOFException: if this stream reaches the end before reading
                      all the bytes.
        - IOException: if an I/O error occurs.
        """
        ...


    def readUnsignedByte(self) -> int:
        """
        Reads one input byte, zero-extends
        it to type `int`, and returns
        the result, which is therefore in the range
        `0`
        through `255`.
        This method is suitable for reading
        the byte written by the `writeByte`
        method of interface `DataOutput`
        if the argument to `writeByte`
        was intended to be a value in the range
        `0` through `255`.

        Returns
        - the unsigned 8-bit value read.

        Raises
        - EOFException: if this stream reaches the end before reading
                      all the bytes.
        - IOException: if an I/O error occurs.
        """
        ...


    def readShort(self) -> int:
        """
        Reads two input bytes and returns
        a `short` value. Let `a`
        be the first byte read and `b`
        be the second byte. The value
        returned
        is:
        ````(short)((a << 8) | (b & 0xff))````
        This method
        is suitable for reading the bytes written
        by the `writeShort` method of
        interface `DataOutput`.

        Returns
        - the 16-bit value read.

        Raises
        - EOFException: if this stream reaches the end before reading
                      all the bytes.
        - IOException: if an I/O error occurs.
        """
        ...


    def readUnsignedShort(self) -> int:
        """
        Reads two input bytes and returns
        an `int` value in the range `0`
        through `65535`. Let `a`
        be the first byte read and
        `b`
        be the second byte. The value returned is:
        ````(((a & 0xff) << 8) | (b & 0xff))````
        This method is suitable for reading the bytes
        written by the `writeShort` method
        of interface `DataOutput`  if
        the argument to `writeShort`
        was intended to be a value in the range
        `0` through `65535`.

        Returns
        - the unsigned 16-bit value read.

        Raises
        - EOFException: if this stream reaches the end before reading
                      all the bytes.
        - IOException: if an I/O error occurs.
        """
        ...


    def readChar(self) -> str:
        """
        Reads two input bytes and returns a `char` value.
        Let `a`
        be the first byte read and `b`
        be the second byte. The value
        returned is:
        ````(char)((a << 8) | (b & 0xff))````
        This method
        is suitable for reading bytes written by
        the `writeChar` method of interface
        `DataOutput`.

        Returns
        - the `char` value read.

        Raises
        - EOFException: if this stream reaches the end before reading
                      all the bytes.
        - IOException: if an I/O error occurs.
        """
        ...


    def readInt(self) -> int:
        """
        Reads four input bytes and returns an
        `int` value. Let `a-d`
        be the first through fourth bytes read. The value returned is:
        ````(((a & 0xff) << 24) | ((b & 0xff) << 16) |
         ((c & 0xff) <<  8) | (d & 0xff))````
        This method is suitable
        for reading bytes written by the `writeInt`
        method of interface `DataOutput`.

        Returns
        - the `int` value read.

        Raises
        - EOFException: if this stream reaches the end before reading
                      all the bytes.
        - IOException: if an I/O error occurs.
        """
        ...


    def readLong(self) -> int:
        """
        Reads eight input bytes and returns
        a `long` value. Let `a-h`
        be the first through eighth bytes read.
        The value returned is:
        ````(((long)(a & 0xff) << 56) |
         ((long)(b & 0xff) << 48) |
         ((long)(c & 0xff) << 40) |
         ((long)(d & 0xff) << 32) |
         ((long)(e & 0xff) << 24) |
         ((long)(f & 0xff) << 16) |
         ((long)(g & 0xff) <<  8) |
         ((long)(h & 0xff)))````
        
        This method is suitable
        for reading bytes written by the `writeLong`
        method of interface `DataOutput`.

        Returns
        - the `long` value read.

        Raises
        - EOFException: if this stream reaches the end before reading
                      all the bytes.
        - IOException: if an I/O error occurs.
        """
        ...


    def readFloat(self) -> float:
        """
        Reads four input bytes and returns
        a `float` value. It does this
        by first constructing an `int`
        value in exactly the manner
        of the `readInt`
        method, then converting this `int`
        value to a `float` in
        exactly the manner of the method `Float.intBitsToFloat`.
        This method is suitable for reading
        bytes written by the `writeFloat`
        method of interface `DataOutput`.

        Returns
        - the `float` value read.

        Raises
        - EOFException: if this stream reaches the end before reading
                      all the bytes.
        - IOException: if an I/O error occurs.
        """
        ...


    def readDouble(self) -> float:
        """
        Reads eight input bytes and returns
        a `double` value. It does this
        by first constructing a `long`
        value in exactly the manner
        of the `readLong`
        method, then converting this `long`
        value to a `double` in exactly
        the manner of the method `Double.longBitsToDouble`.
        This method is suitable for reading
        bytes written by the `writeDouble`
        method of interface `DataOutput`.

        Returns
        - the `double` value read.

        Raises
        - EOFException: if this stream reaches the end before reading
                      all the bytes.
        - IOException: if an I/O error occurs.
        """
        ...


    def readLine(self) -> str:
        """
        Reads the next line of text from the input stream.
        It reads successive bytes, converting
        each byte separately into a character,
        until it encounters a line terminator or
        end of
        file; the characters read are then
        returned as a `String`. Note
        that because this
        method processes bytes,
        it does not support input of the full Unicode
        character set.
        
        If end of file is encountered
        before even one byte can be read, then `null`
        is returned. Otherwise, each byte that is
        read is converted to type `char`
        by zero-extension. If the character `'\n'`
        is encountered, it is discarded and reading
        ceases. If the character `'\r'`
        is encountered, it is discarded and, if
        the following byte converts &#32;to the
        character `'\n'`, then that is
        discarded also; reading then ceases. If
        end of file is encountered before either
        of the characters `'\n'` and
        `'\r'` is encountered, reading
        ceases. Once reading has ceased, a `String`
        is returned that contains all the characters
        read and not discarded, taken in order.
        Note that every character in this string
        will have a value less than `\u005Cu0100`,
        that is, `(char)256`.

        Returns
        - the next line of text from the input stream,
                or `null` if the end of file is
                encountered before a byte can be read.

        Raises
        - IOException: if an I/O error occurs.
        """
        ...


    def readUTF(self) -> str:
        """
        Reads in a string that has been encoded using a
        <a href="#modified-utf-8">modified UTF-8</a>
        format.
        The general contract of `readUTF`
        is that it reads a representation of a Unicode
        character string encoded in modified
        UTF-8 format; this string of characters
        is then returned as a `String`.
        
        First, two bytes are read and used to
        construct an unsigned 16-bit integer in
        exactly the manner of the `readUnsignedShort`
        method . This integer value is called the
        *UTF length* and specifies the number
        of additional bytes to be read. These bytes
        are then converted to characters by considering
        them in groups. The length of each group
        is computed from the value of the first
        byte of the group. The byte following a
        group, if any, is the first byte of the
        next group.
        
        If the first byte of a group
        matches the bit pattern `0xxxxxxx`
        (where `x` means "may be `0`
        or `1`"), then the group consists
        of just that byte. The byte is zero-extended
        to form a character.
        
        If the first byte
        of a group matches the bit pattern `110xxxxx`,
        then the group consists of that byte `a`
        and a second byte `b`. If there
        is no byte `b` (because byte
        `a` was the last of the bytes
        to be read), or if byte `b` does
        not match the bit pattern `10xxxxxx`,
        then a `UTFDataFormatException`
        is thrown. Otherwise, the group is converted
        to the character:
        ````(char)(((a & 0x1F) << 6) | (b & 0x3F))````
        If the first byte of a group
        matches the bit pattern `1110xxxx`,
        then the group consists of that byte `a`
        and two more bytes `b` and `c`.
        If there is no byte `c` (because
        byte `a` was one of the last
        two of the bytes to be read), or either
        byte `b` or byte `c`
        does not match the bit pattern `10xxxxxx`,
        then a `UTFDataFormatException`
        is thrown. Otherwise, the group is converted
        to the character:
        ````(char)(((a & 0x0F) << 12) | ((b & 0x3F) << 6) | (c & 0x3F))````
        If the first byte of a group matches the
        pattern `1111xxxx` or the pattern
        `10xxxxxx`, then a `UTFDataFormatException`
        is thrown.
        
        If end of file is encountered
        at any time during this entire process,
        then an `EOFException` is thrown.
        
        After every group has been converted to
        a character by this process, the characters
        are gathered, in the same order in which
        their corresponding groups were read from
        the input stream, to form a `String`,
        which is returned.
        
        The `writeUTF`
        method of interface `DataOutput`
        may be used to write data that is suitable
        for reading by this method.

        Returns
        - a Unicode string.

        Raises
        - EOFException: if this stream reaches the end
                      before reading all the bytes.
        - IOException: if an I/O error occurs.
        - UTFDataFormatException: if the bytes do not represent a
                      valid modified UTF-8 encoding of a string.
        """
        ...



"""
Python module generated from Java source file com.google.common.math.PairedStats

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtIncompatible
from com.google.common.annotations import J2ktIncompatible
from com.google.common.base import MoreObjects
from com.google.common.base import Objects
from com.google.common.math import *
from java.io import Serializable
from javax.annotation import CheckForNull
from typing import Any, Callable, Iterable, Tuple


class PairedStats(Serializable):
    """
    An immutable value object capturing some basic statistics about a collection of paired double
    values (e.g. points on a plane). Build instances with PairedStatsAccumulator.snapshot.

    Author(s)
    - Pete Gillin

    Since
    - 20.0
    """

    def count(self) -> int:
        """
        Returns the number of pairs in the dataset.
        """
        ...


    def xStats(self) -> "Stats":
        """
        Returns the statistics on the `x` values alone.
        """
        ...


    def yStats(self) -> "Stats":
        """
        Returns the statistics on the `y` values alone.
        """
        ...


    def populationCovariance(self) -> float:
        """
        Returns the population covariance of the values. The count must be non-zero.
        
        This is guaranteed to return zero if the dataset contains a single pair of finite values. It
        is not guaranteed to return zero when the dataset consists of the same pair of values multiple
        times, due to numerical errors.
        
        <h3>Non-finite values</h3>
        
        If the dataset contains any non-finite values (Double.POSITIVE_INFINITY, Double.NEGATIVE_INFINITY, or Double.NaN) then the result is Double.NaN.

        Raises
        - IllegalStateException: if the dataset is empty
        """
        ...


    def sampleCovariance(self) -> float:
        """
        Returns the sample covariance of the values. The count must be greater than one.
        
        This is not guaranteed to return zero when the dataset consists of the same pair of values
        multiple times, due to numerical errors.
        
        <h3>Non-finite values</h3>
        
        If the dataset contains any non-finite values (Double.POSITIVE_INFINITY, Double.NEGATIVE_INFINITY, or Double.NaN) then the result is Double.NaN.

        Raises
        - IllegalStateException: if the dataset is empty or contains a single pair of values
        """
        ...


    def pearsonsCorrelationCoefficient(self) -> float:
        """
        Returns the <a href="http://mathworld.wolfram.com/CorrelationCoefficient.html">Pearson's or
        product-moment correlation coefficient</a> of the values. The count must greater than one, and
        the `x` and `y` values must both have non-zero population variance (i.e. `xStats().populationVariance() > 0.0 && yStats().populationVariance() > 0.0`). The result is not
        guaranteed to be exactly +/-1 even when the data are perfectly (anti-)correlated, due to
        numerical errors. However, it is guaranteed to be in the inclusive range [-1, +1].
        
        <h3>Non-finite values</h3>
        
        If the dataset contains any non-finite values (Double.POSITIVE_INFINITY, Double.NEGATIVE_INFINITY, or Double.NaN) then the result is Double.NaN.

        Raises
        - IllegalStateException: if the dataset is empty or contains a single pair of values, or
            either the `x` and `y` dataset has zero population variance
        """
        ...


    def leastSquaresFit(self) -> "LinearTransformation":
        """
        Returns a linear transformation giving the best fit to the data according to <a
        href="http://mathworld.wolfram.com/LeastSquaresFitting.html">Ordinary Least Squares linear
        regression</a> of `y` as a function of `x`. The count must be greater than one, and
        either the `x` or `y` data must have a non-zero population variance (i.e. `xStats().populationVariance() > 0.0 || yStats().populationVariance() > 0.0`). The result is
        guaranteed to be horizontal if there is variance in the `x` data but not the `y`
        data, and vertical if there is variance in the `y` data but not the `x` data.
        
        This fit minimizes the root-mean-square error in `y` as a function of `x`. This
        error is defined as the square root of the mean of the squares of the differences between the
        actual `y` values of the data and the values predicted by the fit for the `x`
        values (i.e. it is the square root of the mean of the squares of the vertical distances between
        the data points and the best fit line). For this fit, this error is a fraction `sqrt(1 -
        R*R)` of the population standard deviation of `y`, where `R` is the Pearson's
        correlation coefficient (as given by .pearsonsCorrelationCoefficient()).
        
        The corresponding root-mean-square error in `x` as a function of `y` is a
        fraction `sqrt(1/(R*R) - 1)` of the population standard deviation of `x`. This fit
        does not normally minimize that error: to do that, you should swap the roles of `x` and
        `y`.
        
        <h3>Non-finite values</h3>
        
        If the dataset contains any non-finite values (Double.POSITIVE_INFINITY, Double.NEGATIVE_INFINITY, or Double.NaN) then the result is LinearTransformation.forNaN().

        Raises
        - IllegalStateException: if the dataset is empty or contains a single pair of values, or
            both the `x` and `y` dataset must have zero population variance
        """
        ...


    def equals(self, obj: "Object") -> bool:
        """
        
        
        **Note:** This tests exact equality of the calculated statistics, including the floating
        point values. Two instances are guaranteed to be considered equal if one is copied from the
        other using `second = new PairedStatsAccumulator().addAll(first).snapshot()`, if both
        were obtained by calling `snapshot()` on the same PairedStatsAccumulator without
        adding any values in between the two calls, or if one is obtained from the other after
        round-tripping through java serialization. However, floating point rounding errors mean that it
        may be False for some instances where the statistics are mathematically equal, including
        instances constructed from the same values in a different order... or (in the general case)
        even in the same order. (It is guaranteed to return True for instances constructed from the
        same values in the same order if `strictfp` is in effect, or if the system architecture
        guarantees `strictfp`-like semantics.)
        """
        ...


    def hashCode(self) -> int:
        """
        
        
        **Note:** This hash code is consistent with exact equality of the calculated statistics,
        including the floating point values. See the note on .equals for details.
        """
        ...


    def toString(self) -> str:
        ...


    def toByteArray(self) -> list[int]:
        """
        Gets a byte array representation of this instance.
        
        **Note:** No guarantees are made regarding stability of the representation between
        versions.
        """
        ...


    @staticmethod
    def fromByteArray(byteArray: list[int]) -> "PairedStats":
        """
        Creates a PairedStats instance from the given byte representation which was obtained by
        .toByteArray.
        
        **Note:** No guarantees are made regarding stability of the representation between
        versions.
        """
        ...



"""
Python module generated from Java source file java.io.ObjectOutputStream

Java source file obtained from artifact jdk version jdk

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from java.io import *
from java.io.ObjectStreamClass import WeakClassKey
from java.lang.ref import ReferenceQueue
from java.security import AccessController
from java.security import PrivilegedAction
from java.util import Arrays
from java.util import StringJoiner
from java.util.concurrent import ConcurrentHashMap
from java.util.concurrent import ConcurrentMap
from sun.reflect.misc import ReflectUtil
from typing import Any, Callable, Iterable, Tuple


class ObjectOutputStream(OutputStream, ObjectOutput, ObjectStreamConstants):
    """
    An ObjectOutputStream writes primitive data types and graphs of Java objects
    to an OutputStream.  The objects can be read (reconstituted) using an
    ObjectInputStream.  Persistent storage of objects can be accomplished by
    using a file for the stream.  If the stream is a network socket stream, the
    objects can be reconstituted on another host or in another process.
    
    Only objects that support the java.io.Serializable interface can be
    written to streams.  The class of each serializable object is encoded
    including the class name and signature of the class, the values of the
    object's fields and arrays, and the closure of any other objects referenced
    from the initial objects.
    
    The method writeObject is used to write an object to the stream.  Any
    object, including Strings and arrays, is written with writeObject. Multiple
    objects or primitives can be written to the stream.  The objects must be
    read back from the corresponding ObjectInputstream with the same types and
    in the same order as they were written.
    
    Primitive data types can also be written to the stream using the
    appropriate methods from DataOutput. Strings can also be written using the
    writeUTF method.
    
    The default serialization mechanism for an object writes the class of the
    object, the class signature, and the values of all non-transient and
    non-static fields.  References to other objects (except in transient or
    static fields) cause those objects to be written also. Multiple references
    to a single object are encoded using a reference sharing mechanism so that
    graphs of objects can be restored to the same shape as when the original was
    written.
    
    For example to write an object that can be read by the example in
    ObjectInputStream:
    
    ```
         FileOutputStream fos = new FileOutputStream("t.tmp");
         ObjectOutputStream oos = new ObjectOutputStream(fos);
    
         oos.writeInt(12345);
         oos.writeObject("Today");
         oos.writeObject(new Date());
    
         oos.close();
    ```
    
    Classes that require special handling during the serialization and
    deserialization process must implement special methods with these exact
    signatures:
    
    ```
    private void readObject(java.io.ObjectInputStream stream)
        throws IOException, ClassNotFoundException;
    private void writeObject(java.io.ObjectOutputStream stream)
        throws IOException
    private void readObjectNoData()
        throws ObjectStreamException;
    ```
    
    The writeObject method is responsible for writing the state of the object
    for its particular class so that the corresponding readObject method can
    restore it.  The method does not need to concern itself with the state
    belonging to the object's superclasses or subclasses.  State is saved by
    writing the individual fields to the ObjectOutputStream using the
    writeObject method or by using the methods for primitive data types
    supported by DataOutput.
    
    Serialization does not write out the fields of any object that does not
    implement the java.io.Serializable interface.  Subclasses of Objects that
    are not serializable can be serializable. In this case the non-serializable
    class must have a no-arg constructor to allow its fields to be initialized.
    In this case it is the responsibility of the subclass to save and restore
    the state of the non-serializable class. It is frequently the case that the
    fields of that class are accessible (public, package, or protected) or that
    there are get and set methods that can be used to restore the state.
    
    Serialization of an object can be prevented by implementing writeObject
    and readObject methods that throw the NotSerializableException.  The
    exception will be caught by the ObjectOutputStream and abort the
    serialization process.
    
    Implementing the Externalizable interface allows the object to assume
    complete control over the contents and format of the object's serialized
    form.  The methods of the Externalizable interface, writeExternal and
    readExternal, are called to save and restore the objects state.  When
    implemented by a class they can write and read their own state using all of
    the methods of ObjectOutput and ObjectInput.  It is the responsibility of
    the objects to handle any versioning that occurs.
    
    Enum constants are serialized differently than ordinary serializable or
    externalizable objects.  The serialized form of an enum constant consists
    solely of its name; field values of the constant are not transmitted.  To
    serialize an enum constant, ObjectOutputStream writes the string returned by
    the constant's name method.  Like other serializable or externalizable
    objects, enum constants can function as the targets of back references
    appearing subsequently in the serialization stream.  The process by which
    enum constants are serialized cannot be customized; any class-specific
    writeObject and writeReplace methods defined by enum types are ignored
    during serialization.  Similarly, any serialPersistentFields or
    serialVersionUID field declarations are also ignored--all enum types have a
    fixed serialVersionUID of 0L.
    
    Primitive data, excluding serializable fields and externalizable data, is
    written to the ObjectOutputStream in block-data records. A block data record
    is composed of a header and data. The block data header consists of a marker
    and the number of bytes to follow the header.  Consecutive primitive data
    writes are merged into one block-data record.  The blocking factor used for
    a block-data record will be 1024 bytes.  Each block-data record will be
    filled up to 1024 bytes, or be written whenever there is a termination of
    block-data mode.  Calls to the ObjectOutputStream methods writeObject,
    defaultWriteObject and writeFields initially terminate any existing
    block-data record.
    
    Records are serialized differently than ordinary serializable or externalizable
    objects, see <a href="ObjectInputStream.html#record-serialization">record serialization</a>.

    Author(s)
    - Roger Riggs

    See
    - <a href="/../specs/serialization/output.html">
         <cite>Java Object Serialization Specification,</cite> Section 2, "Object Output Classes"</a>

    Since
    - 1.1
    """

    def __init__(self, out: "OutputStream"):
        """
        Creates an ObjectOutputStream that writes to the specified OutputStream.
        This constructor writes the serialization stream header to the
        underlying stream; callers may wish to flush the stream immediately to
        ensure that constructors for receiving ObjectInputStreams will not block
        when reading the header.
        
        If a security manager is installed, this constructor will check for
        the "enableSubclassImplementation" SerializablePermission when invoked
        directly or indirectly by the constructor of a subclass which overrides
        the ObjectOutputStream.putFields or ObjectOutputStream.writeUnshared
        methods.

        Arguments
        - out: output stream to write to

        Raises
        - IOException: if an I/O error occurs while writing stream header
        - SecurityException: if untrusted subclass illegally overrides
                 security-sensitive methods
        - NullPointerException: if `out` is `null`

        See
        - ObjectInputStream.ObjectInputStream(InputStream)

        Since
        - 1.4
        """
        ...


    def useProtocolVersion(self, version: int) -> None:
        """
        Specify stream protocol version to use when writing the stream.
        
        This routine provides a hook to enable the current version of
        Serialization to write in a format that is backwards compatible to a
        previous version of the stream format.
        
        Every effort will be made to avoid introducing additional
        backwards incompatibilities; however, sometimes there is no
        other alternative.

        Arguments
        - version: use ProtocolVersion from java.io.ObjectStreamConstants.

        Raises
        - IllegalStateException: if called after any objects
                 have been serialized.
        - IllegalArgumentException: if invalid version is passed in.
        - IOException: if I/O errors occur

        See
        - java.io.ObjectStreamConstants.PROTOCOL_VERSION_2

        Since
        - 1.2
        """
        ...


    def writeObject(self, obj: "Object") -> None:
        """
        Write the specified object to the ObjectOutputStream.  The class of the
        object, the signature of the class, and the values of the non-transient
        and non-static fields of the class and all of its supertypes are
        written.  Default serialization for a class can be overridden using the
        writeObject and the readObject methods.  Objects referenced by this
        object are written transitively so that a complete equivalent graph of
        objects can be reconstructed by an ObjectInputStream.
        
        Exceptions are thrown for problems with the OutputStream and for
        classes that should not be serialized.  All exceptions are fatal to the
        OutputStream, which is left in an indeterminate state, and it is up to
        the caller to ignore or recover the stream state.

        Raises
        - InvalidClassException: Something is wrong with a class used by
                 serialization.
        - NotSerializableException: Some object to be serialized does not
                 implement the java.io.Serializable interface.
        - IOException: Any exception thrown by the underlying
                 OutputStream.
        """
        ...


    def writeUnshared(self, obj: "Object") -> None:
        """
        Writes an "unshared" object to the ObjectOutputStream.  This method is
        identical to writeObject, except that it always writes the given object
        as a new, unique object in the stream (as opposed to a back-reference
        pointing to a previously serialized instance).  Specifically:
        
          - An object written via writeUnshared is always serialized in the
              same manner as a newly appearing object (an object that has not
              been written to the stream yet), regardless of whether or not the
              object has been written previously.
        
          - If writeObject is used to write an object that has been previously
              written with writeUnshared, the previous writeUnshared operation
              is treated as if it were a write of a separate object.  In other
              words, ObjectOutputStream will never generate back-references to
              object data written by calls to writeUnshared.
        
        While writing an object via writeUnshared does not in itself guarantee a
        unique reference to the object when it is deserialized, it allows a
        single object to be defined multiple times in a stream, so that multiple
        calls to readUnshared by the receiver will not conflict.  Note that the
        rules described above only apply to the base-level object written with
        writeUnshared, and not to any transitively referenced sub-objects in the
        object graph to be serialized.
        
        ObjectOutputStream subclasses which override this method can only be
        constructed in security contexts possessing the
        "enableSubclassImplementation" SerializablePermission; any attempt to
        instantiate such a subclass without this permission will cause a
        SecurityException to be thrown.

        Arguments
        - obj: object to write to stream

        Raises
        - NotSerializableException: if an object in the graph to be
                 serialized does not implement the Serializable interface
        - InvalidClassException: if a problem exists with the class of an
                 object to be serialized
        - IOException: if an I/O error occurs during serialization

        Since
        - 1.4
        """
        ...


    def defaultWriteObject(self) -> None:
        """
        Write the non-static and non-transient fields of the current class to
        this stream.  This may only be called from the writeObject method of the
        class being serialized. It will throw the NotActiveException if it is
        called otherwise.

        Raises
        - IOException: if I/O errors occur while writing to the underlying
                 `OutputStream`
        """
        ...


    def putFields(self) -> "ObjectOutputStream.PutField":
        """
        Retrieve the object used to buffer persistent fields to be written to
        the stream.  The fields will be written to the stream when writeFields
        method is called.

        Returns
        - an instance of the class Putfield that holds the serializable
                 fields

        Raises
        - IOException: if I/O errors occur

        Since
        - 1.2
        """
        ...


    def writeFields(self) -> None:
        """
        Write the buffered fields to the stream.

        Raises
        - IOException: if I/O errors occur while writing to the underlying
                 stream
        - NotActiveException: Called when a classes writeObject method was
                 not called to write the state of the object.

        Since
        - 1.2
        """
        ...


    def reset(self) -> None:
        """
        Reset will disregard the state of any objects already written to the
        stream.  The state is reset to be the same as a new ObjectOutputStream.
        The current point in the stream is marked as reset so the corresponding
        ObjectInputStream will be reset at the same point.  Objects previously
        written to the stream will not be referred to as already being in the
        stream.  They will be written to the stream again.

        Raises
        - IOException: if reset() is invoked while serializing an object.
        """
        ...


    def write(self, val: int) -> None:
        """
        Writes a byte. This method will block until the byte is actually
        written.

        Arguments
        - val: the byte to be written to the stream

        Raises
        - IOException: If an I/O error has occurred.
        """
        ...


    def write(self, buf: list[int]) -> None:
        """
        Writes an array of bytes. This method will block until the bytes are
        actually written.

        Arguments
        - buf: the data to be written

        Raises
        - IOException: If an I/O error has occurred.
        """
        ...


    def write(self, buf: list[int], off: int, len: int) -> None:
        """
        Writes a sub array of bytes.

        Arguments
        - buf: the data to be written
        - off: the start offset in the data
        - len: the number of bytes that are written

        Raises
        - IOException: If an I/O error has occurred.
        """
        ...


    def flush(self) -> None:
        """
        Flushes the stream. This will write any buffered output bytes and flush
        through to the underlying stream.

        Raises
        - IOException: If an I/O error has occurred.
        """
        ...


    def close(self) -> None:
        """
        Closes the stream. This method must be called to release any resources
        associated with the stream.

        Raises
        - IOException: If an I/O error has occurred.
        """
        ...


    def writeBoolean(self, val: bool) -> None:
        """
        Writes a boolean.

        Arguments
        - val: the boolean to be written

        Raises
        - IOException: if I/O errors occur while writing to the underlying
                 stream
        """
        ...


    def writeByte(self, val: int) -> None:
        """
        Writes an 8 bit byte.

        Arguments
        - val: the byte value to be written

        Raises
        - IOException: if I/O errors occur while writing to the underlying
                 stream
        """
        ...


    def writeShort(self, val: int) -> None:
        """
        Writes a 16 bit short.

        Arguments
        - val: the short value to be written

        Raises
        - IOException: if I/O errors occur while writing to the underlying
                 stream
        """
        ...


    def writeChar(self, val: int) -> None:
        """
        Writes a 16 bit char.

        Arguments
        - val: the char value to be written

        Raises
        - IOException: if I/O errors occur while writing to the underlying
                 stream
        """
        ...


    def writeInt(self, val: int) -> None:
        """
        Writes a 32 bit int.

        Arguments
        - val: the integer value to be written

        Raises
        - IOException: if I/O errors occur while writing to the underlying
                 stream
        """
        ...


    def writeLong(self, val: int) -> None:
        """
        Writes a 64 bit long.

        Arguments
        - val: the long value to be written

        Raises
        - IOException: if I/O errors occur while writing to the underlying
                 stream
        """
        ...


    def writeFloat(self, val: float) -> None:
        """
        Writes a 32 bit float.

        Arguments
        - val: the float value to be written

        Raises
        - IOException: if I/O errors occur while writing to the underlying
                 stream
        """
        ...


    def writeDouble(self, val: float) -> None:
        """
        Writes a 64 bit double.

        Arguments
        - val: the double value to be written

        Raises
        - IOException: if I/O errors occur while writing to the underlying
                 stream
        """
        ...


    def writeBytes(self, str: str) -> None:
        """
        Writes a String as a sequence of bytes.

        Arguments
        - str: the String of bytes to be written

        Raises
        - IOException: if I/O errors occur while writing to the underlying
                 stream
        """
        ...


    def writeChars(self, str: str) -> None:
        """
        Writes a String as a sequence of chars.

        Arguments
        - str: the String of chars to be written

        Raises
        - IOException: if I/O errors occur while writing to the underlying
                 stream
        """
        ...


    def writeUTF(self, str: str) -> None:
        """
        Primitive data write of this String in
        <a href="DataInput.html#modified-utf-8">modified UTF-8</a>
        format.  Note that there is a
        significant difference between writing a String into the stream as
        primitive data or as an Object. A String instance written by writeObject
        is written into the stream as a String initially. Future writeObject()
        calls write references to the string into the stream.

        Arguments
        - str: the String to be written

        Raises
        - IOException: if I/O errors occur while writing to the underlying
                 stream
        """
        ...


    class PutField:
        """
        Provide programmatic access to the persistent fields to be written
        to ObjectOutput.

        Since
        - 1.2
        """

        def __init__(self):
            """
            Constructor for subclasses to call.
            """
            ...


        def put(self, name: str, val: bool) -> None:
            """
            Put the value of the named boolean field into the persistent field.

            Arguments
            - name: the name of the serializable field
            - val: the value to assign to the field

            Raises
            - IllegalArgumentException: if `name` does not
            match the name of a serializable field for the class whose fields
            are being written, or if the type of the named field is not
            `boolean`
            """
            ...


        def put(self, name: str, val: int) -> None:
            """
            Put the value of the named byte field into the persistent field.

            Arguments
            - name: the name of the serializable field
            - val: the value to assign to the field

            Raises
            - IllegalArgumentException: if `name` does not
            match the name of a serializable field for the class whose fields
            are being written, or if the type of the named field is not
            `byte`
            """
            ...


        def put(self, name: str, val: str) -> None:
            """
            Put the value of the named char field into the persistent field.

            Arguments
            - name: the name of the serializable field
            - val: the value to assign to the field

            Raises
            - IllegalArgumentException: if `name` does not
            match the name of a serializable field for the class whose fields
            are being written, or if the type of the named field is not
            `char`
            """
            ...


        def put(self, name: str, val: int) -> None:
            """
            Put the value of the named short field into the persistent field.

            Arguments
            - name: the name of the serializable field
            - val: the value to assign to the field

            Raises
            - IllegalArgumentException: if `name` does not
            match the name of a serializable field for the class whose fields
            are being written, or if the type of the named field is not
            `short`
            """
            ...


        def put(self, name: str, val: int) -> None:
            """
            Put the value of the named int field into the persistent field.

            Arguments
            - name: the name of the serializable field
            - val: the value to assign to the field

            Raises
            - IllegalArgumentException: if `name` does not
            match the name of a serializable field for the class whose fields
            are being written, or if the type of the named field is not
            `int`
            """
            ...


        def put(self, name: str, val: int) -> None:
            """
            Put the value of the named long field into the persistent field.

            Arguments
            - name: the name of the serializable field
            - val: the value to assign to the field

            Raises
            - IllegalArgumentException: if `name` does not
            match the name of a serializable field for the class whose fields
            are being written, or if the type of the named field is not
            `long`
            """
            ...


        def put(self, name: str, val: float) -> None:
            """
            Put the value of the named float field into the persistent field.

            Arguments
            - name: the name of the serializable field
            - val: the value to assign to the field

            Raises
            - IllegalArgumentException: if `name` does not
            match the name of a serializable field for the class whose fields
            are being written, or if the type of the named field is not
            `float`
            """
            ...


        def put(self, name: str, val: float) -> None:
            """
            Put the value of the named double field into the persistent field.

            Arguments
            - name: the name of the serializable field
            - val: the value to assign to the field

            Raises
            - IllegalArgumentException: if `name` does not
            match the name of a serializable field for the class whose fields
            are being written, or if the type of the named field is not
            `double`
            """
            ...


        def put(self, name: str, val: "Object") -> None:
            """
            Put the value of the named Object field into the persistent field.

            Arguments
            - name: the name of the serializable field
            - val: the value to assign to the field
                    (which may be `null`)

            Raises
            - IllegalArgumentException: if `name` does not
            match the name of a serializable field for the class whose fields
            are being written, or if the type of the named field is not a
            reference type
            """
            ...


        def write(self, out: "ObjectOutput") -> None:
            """
            Write the data and fields to the specified ObjectOutput stream,
            which must be the same stream that produced this
            `PutField` object.

            Arguments
            - out: the stream to write the data and fields to

            Raises
            - IOException: if I/O errors occur while writing to the
                    underlying stream
            - IllegalArgumentException: if the specified stream is not
                    the same stream that produced this `PutField`
                    object

            Deprecated
            - This method does not write the values contained by this
                    `PutField` object in a proper format, and may
                    result in corruption of the serialization stream.  The
                    correct way to write `PutField` data is by
                    calling the java.io.ObjectOutputStream.writeFields()
                    method.
            """
            ...



"""
Python module generated from Java source file org.bukkit.permissions.PermissionDefault

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from enum import Enum
from java.util import Locale
from org.bukkit.permissions import *
from typing import Any, Callable, Iterable, Tuple


class PermissionDefault(Enum):
    """
    Represents the possible default values for permissions
    """

    TRUE = ("true")
    FALSE = ("false")
    OP = ("op", "isop", "operator", "isoperator", "admin", "isadmin")
    NOT_OP = ("!op", "notop", "!operator", "notoperator", "!admin", "notadmin")


    def getValue(self, op: bool) -> bool:
        """
        Calculates the value of this PermissionDefault for the given operator
        value

        Arguments
        - op: If the target is op

        Returns
        - True if the default should be True, or False
        """
        ...


    @staticmethod
    def getByName(name: str) -> "PermissionDefault":
        """
        Looks up a PermissionDefault by name

        Arguments
        - name: Name of the default

        Returns
        - Specified value, or null if not found
        """
        ...


    def toString(self) -> str:
        ...



"""
Python module generated from Java source file org.bukkit.event.enchantment.EnchantItemEvent

Java source file obtained from artifact spigot-api version 1.21.4-R0.1-20250303.102353-42

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.base import Preconditions
from org.bukkit.block import Block
from org.bukkit.enchantments import Enchantment
from org.bukkit.entity import Player
from org.bukkit.event import Cancellable
from org.bukkit.event import HandlerList
from org.bukkit.event.enchantment import *
from org.bukkit.event.inventory import InventoryEvent
from org.bukkit.inventory import InventoryView
from org.bukkit.inventory import ItemStack
from typing import Any, Callable, Iterable, Tuple


class EnchantItemEvent(InventoryEvent, Cancellable):
    """
    Called when an ItemStack is successfully enchanted (currently at
    enchantment table)
    """

    def __init__(self, enchanter: "Player", view: "InventoryView", table: "Block", item: "ItemStack", level: int, enchants: dict["Enchantment", "Integer"], enchantmentHint: "Enchantment", levelHint: int, i: int):
        ...


    def getEnchanter(self) -> "Player":
        """
        Gets the player enchanting the item

        Returns
        - enchanting player
        """
        ...


    def getEnchantBlock(self) -> "Block":
        """
        Gets the block being used to enchant the item

        Returns
        - the block used for enchanting
        """
        ...


    def getItem(self) -> "ItemStack":
        """
        Gets the item to be enchanted (can be modified)

        Returns
        - ItemStack of item
        """
        ...


    def getExpLevelCost(self) -> int:
        """
        Gets the cost (minimum level) which is displayed as a number on the right
        hand side of the enchantment offer.

        Returns
        - experience level cost
        """
        ...


    def setExpLevelCost(self, level: int) -> None:
        """
        Sets the cost (minimum level) which is displayed as a number on the right
        hand side of the enchantment offer.

        Arguments
        - level: - cost in levels
        """
        ...


    def getEnchantsToAdd(self) -> dict["Enchantment", "Integer"]:
        """
        Get map of enchantment (levels, keyed by type) to be added to item
        (modify map returned to change values). Note: Any enchantments not
        allowed for the item will be ignored

        Returns
        - map of enchantment levels, keyed by enchantment
        """
        ...


    def getEnchantmentHint(self) -> "Enchantment":
        """
        Get the Enchantment that was displayed as a hint to the player
        on the selected enchantment offer.

        Returns
        - the hinted enchantment
        """
        ...


    def getLevelHint(self) -> int:
        """
        Get the level of the enchantment that was displayed as a hint to the
        player on the selected enchantment offer.

        Returns
        - the level of the hinted enchantment
        """
        ...


    def whichButton(self) -> int:
        """
        Which button was pressed to initiate the enchanting.

        Returns
        - The button index (0, 1, or 2).
        """
        ...


    def isCancelled(self) -> bool:
        ...


    def setCancelled(self, cancel: bool) -> None:
        ...


    def getHandlers(self) -> "HandlerList":
        ...


    @staticmethod
    def getHandlerList() -> "HandlerList":
        ...



"""
Python module generated from Java source file com.google.common.collect.UnmodifiableListIterator

Java source file obtained from artifact guava version 33.3.1-jre

Because this Python module is automatically generated, it may contain errors
and/or code that cannot be parsed. Please report these issues at
https://github.com/magicmq/docs-translator/issues
"""
from com.google.common.annotations import GwtCompatible
from com.google.common.collect import *
from com.google.errorprone.annotations import DoNotCall
from java.util import ListIterator
from org.checkerframework.checker.nullness.qual import Nullable
from typing import Any, Callable, Iterable, Tuple


class UnmodifiableListIterator(UnmodifiableIterator, ListIterator):
    """
    A list iterator that does not support .remove, .add, or .set.

    Author(s)
    - Louis Wasserman

    Since
    - 7.0
    """

    def add(self, e: "E") -> None:
        """
        Guaranteed to throw an exception and leave the underlying data unmodified.

        Raises
        - UnsupportedOperationException: always

        Deprecated
        - Unsupported operation.
        """
        ...


    def set(self, e: "E") -> None:
        """
        Guaranteed to throw an exception and leave the underlying data unmodified.

        Raises
        - UnsupportedOperationException: always

        Deprecated
        - Unsupported operation.
        """
        ...
